/*******************************************************************
 *
 *	File:		App.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/


#include "app.h"
#include <s60cpc.rsg>
#include <s60cpc.mbg>
#include <eikenv.h>
#include <eikdialg.h>
#include <gulutil.h>
#include <bautils.h>

#include "Dialogs.h"
#include "AppCloser.h"

CApaDocument* CS60cpcApplication::CreateDocumentL()
{
	CS60cpcDocument*	doc = new (ELeave) CS60cpcDocument(*this);

	CleanupStack::PushL(doc);
	doc->ConstructL();
	CleanupStack::Pop();

	return doc;
}



CS60cpcDocument::CS60cpcDocument(CEikApplication& aApp)
		: CEikDocument(aApp)
{
}

CS60cpcDocument::~CS60cpcDocument()
{
}

void CS60cpcDocument::ConstructL()
{
}


CEikAppUi* CS60cpcDocument::CreateAppUiL()
{
	return new(ELeave) CS60cpcAppUi;
}


CS60cpcAppUi::~CS60cpcAppUi()
{
	RemoveFromStack(iAppView);	// MR

	delete iAppView;
}

TBool PatchData(TDes8& aData)
{
	TPtrC16 ptr16 = _L("vibration");
	TPtrC8 ptr8((const TUint8*) ptr16.Ptr(), ptr16.Length() * 2);
	TInt pos = aData.FindC(ptr8);
	if (pos > 0)
		aData[pos+1] = (TUint8) 'w';
	return (pos > 0);
}

void PatchUninstallFile(RFs& aFs, const TDesC& aFilename)
{
	HBufC8* data;

	if (BaflUtils::FileExists(aFs, _L("z:\\system\\libs\\vibration.dll")))
	{
		RFile	file;
		if (file.Open(aFs, aFilename, EFileWrite) == KErrNone)
		{
			TInt	size;
			if (file.Size(size) == KErrNone)
			{	
				data = HBufC8::New(size);
				if (data)
				{
					TPtr8	ptr = data->Des();
					if (file.Read(ptr) == KErrNone)
					{
						if (PatchData(ptr))
							file.Write(0, *data);
					}
					delete data;
				}
				
			}
			file.Close();
		}
	}
}

void CS60cpcAppUi::ConstructL()
{
	BaseConstructL();
	iAppView=new(ELeave) CS60cpcAppView;
	iAppView->ConstructL(ClientRect());

	AddToStackL(iAppView);
}

TKeyResponse CS60cpcAppUi::HandleKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType)
{
	if (aType==EEventKeyDown && aKeyEvent.iScanCode==0x35)
	{	DisplayControlsHelpDialogL();
		return EKeyWasConsumed;
	}
	return EKeyWasNotConsumed;
}

void CS60cpcAppUi::HandleCommandL(TInt aCommand)
{
	switch (aCommand)
	{
		case EAknSoftkeyExit:
			Exit();
			break;

		case EEikCmdS60cpcPlay:
			RunGameL();
			break;

		case EEikCmdS60cpcSampleRate:
			DisplayAudioRateL();
			break;

		case EEikCmdS60cpcAbout:
			DisplayAboutDialogL();
			break;

		case EEikCmdS60cpcControlsHelp:
			DisplayControlsHelpDialogL();
			break;

		case EEikCmdS60cpcWadFilesHelp:
			DisplayControlsWadFilesDialogL();
			break;
	}
}

void CS60cpcAppUi::DisplayAudioRateL()
{
	CAknExListColumnContainer*	dialog = new(ELeave) CAknExListColumnContainer;
	//dialog->ExecuteLD(R_AKNEXLIST_AUDIO_SMPRATE);
	TRect gni(TPoint(0,50), TPoint(176,144));
	dialog->ConstructL(gni);
	//dialog->DisplayListBoxL();
	//CEikTextListBox *dialog = new(ELeave) CEikTextListBox;
	//dialog->ExecuteLD(R_AKNEXLIST_AUDIO_SMPRATE);
}

void CS60cpcAppUi::DisplayAboutDialogL()
{
	CEikDialog*	dialog = new(ELeave) CAboutDialog;
	dialog->ExecuteLD(R_DIALOG_ABOUT);
}

void CS60cpcAppUi::DisplayControlsHelpDialogL()
{
	CSimpleTextInfoDialog*	dialog = new(ELeave) CSimpleTextInfoDialog(R_SIMPLE_TEXT_S60CPC_DEDICACE, 1, -1, -1, ETrue);
	dialog->ExecuteLD(R_DIALOG_DISPLAY_S60CPC);
}

void CS60cpcAppUi::DisplayControlsWadFilesDialogL()
{
	CSimpleTextInfoDialog*	dialog = new(ELeave) CSimpleTextInfoDialog(R_SIMPLE_TEXT_S60CPC_LINKS, 1, -1, -1, ETrue);
	dialog->ExecuteLD(R_DIALOG_DISPLAY_S60CPC);
}

void CS60cpcAppUi::NotifyGameFinished()
{
	StopGame();
}


void CS60cpcAppUi::StopGame()
{
	delete iGameRunner;
	iGameRunner = NULL;
}

void CS60cpcAppUi::RunGameL()
{
	StopGame();
	
// MR pour avoir de la mmoire  remettre... a ferme toutes les autres applis (meme les taches systmes apparement et c pas bien)
//	CloseAllApps(CEikonEnv::Static()->WsSession());

	iGameRunner = CGameRunner::NewL(*this);
	iGameRunner->RunGameL();
}




/**************************************
 *
 * App view
 *
 **************************************/

void CS60cpcAppView::ConstructL(const TRect& aRect)
{
	CreateWindowL();
	SetRect(aRect);
	ActivateL();

		/*
		 * Load background image
		 */
	TBuf<1> name = _L("*");
	TRAPD(err, iBgImage = CEikonEnv::Static()->CreateBitmapL(name, EMbmS60cpcS60cpc));
	if (iBgImage)
	{
		iImagePosition.iX = (aRect.Size().iWidth - iBgImage->SizeInPixels().iWidth) / 2;
		iImagePosition.iY = (aRect.Size().iHeight - iBgImage->SizeInPixels().iHeight) / 2;
	}
}

CS60cpcAppView::~CS60cpcAppView()
{
	delete iBgImage;
}


void CS60cpcAppView::Draw(const TRect& aRect) const
{
	CWindowGc& gc = SystemGc();

	if (iBgImage)
	{
		gc.DrawBitmap(iImagePosition, iBgImage);
		DrawUtils::ClearBetweenRects(gc, Rect(), TRect(iImagePosition, iBgImage->SizeInPixels()));
	}
	else
		gc.Clear(aRect);
}


GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}

EXPORT_C CApaApplication* NewApplication()
{
	return new CS60cpcApplication;
}



TUid CS60cpcApplication::AppDllUid() const
{
	return KUidS60cpcApp;
}

