/*******************************************************************
 *
 *	File:		Dialogs.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "Dialogs.h"
#include "Engine.h"
#include "S60cpc.hrh"
#include "S60cpc.rsg"

#include "version.h"
#include "CSimpleTextParser.h"
#include <txtrich.h>				// CRichText
#include <eikrted.h>				// CEikRichTextEditor

#include <barsread.h>
#include <aknlists.h> // for avrell style listbox


/************************************************
 *
 * SimpleTextInfoDialog
 *
 ************************************************/


CSimpleTextInfoDialog::CSimpleTextInfoDialog(TInt aTextIdOne, TInt aRichTextCtlIdOne, TInt aTextIdTwo, TInt aRichTextCtlIdTwo, TBool aSimpleTextResIdOneIsArray, TBool aSimpleTextResIdTwoIsArray)
	: iRichTextCtlIdOne(aRichTextCtlIdOne), iSimpleTextResIdOne(aTextIdOne),
	  iRichTextCtlIdTwo(aRichTextCtlIdTwo), iSimpleTextResIdTwo(aTextIdTwo),
	  iSimpleTextResIdOneIsArray(aSimpleTextResIdOneIsArray), iSimpleTextResIdTwoIsArray(aSimpleTextResIdTwoIsArray),
	  iSetDialogBackground(ETrue)
{
}

void CSimpleTextInfoDialog::PreLayoutDynInitL()
{
	CEikRichTextEditor*		richTextEditor;
	
	if (iRichTextCtlIdOne != -1)
	{
		richTextEditor=STATIC_CAST(CEikRichTextEditor*, Control(iRichTextCtlIdOne));
		PreLayoutDynInitRichTextL(*richTextEditor, iRichTextCtlIdOne, iSimpleTextResIdOne);
	}

	if (iRichTextCtlIdTwo != -1)
	{
		richTextEditor=STATIC_CAST(CEikRichTextEditor*, Control(iRichTextCtlIdTwo));
		richTextEditor->Border().SetType(ENone);
		PreLayoutDynInitRichTextL(*richTextEditor, iRichTextCtlIdTwo, iSimpleTextResIdTwo);
	}
}


void CSimpleTextInfoDialog::PreLayoutDynInitRichTextL(CEikRichTextEditor& aRichTextEditor, TInt aRichTextCtlId, TInt aSimpleTextResId)
{
	iEikonEnv->BusyMsgL(_L("Opening"));
	aRichTextEditor.SetEdwinSizeObserver(this);
	if (iSetDialogBackground)
		aRichTextEditor.SetBackgroundColorL(iEikonEnv->Color(EColorDialogBackground));
	aRichTextEditor.SetSize(aRichTextEditor.Size()); // Set the size of the edwin

		// no scrollbars for us
	aRichTextEditor.CreateScrollBarFrameL(); // Create the scrollbars
	aRichTextEditor.ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, iWantVertScrollbar ? CEikScrollBarFrame::EAuto: CEikScrollBarFrame::EOff);

	ShowTextL(*aRichTextEditor.RichText(), aRichTextCtlId, aSimpleTextResId);

	aRichTextEditor.UpdateAllFieldsL(); // Updates all the fields in the document

	aRichTextEditor.UpdateScrollBarsL();
}


void CSimpleTextInfoDialog::PostLayoutDynInitL()
{
	Layout();
	iEikonEnv->BusyMsgCancel();
}

TBool CSimpleTextInfoDialog::HandleEdwinSizeEventL(CEikEdwin* aEdwin, TEdwinSizeEvent aEventType, TSize aDesirableEdwinSize)
{
	if ((aEventType == EEventSizeChanging))
		aEdwin->SetSize(aDesirableEdwinSize);
	return EFalse;
}

void CSimpleTextInfoDialog::ShowTextL(CRichText& aRichText, TInt /*aRichTextCtlId*/, TInt aResId)
{
	if (aResId != -1)
	{
		if ( ((aResId == iSimpleTextResIdOne) && (iSimpleTextResIdOneIsArray)) ||
		     ((aResId == iSimpleTextResIdTwo) && (iSimpleTextResIdTwoIsArray))
		   )
		{
			CDesCArrayFlat* desArray = CEikonEnv::Static()->ReadDesCArrayResourceL(aResId);
			CleanupStack::PushL(desArray);

			CSimpleTextFormatParser* parser = CSimpleTextFormatParser::NewLC();

			TInt	count = desArray->Count();
			for (TInt i=0 ;i<count ; i++)
				parser->ParseL(desArray->operator[](i), aRichText);

			CleanupStack::PopAndDestroy(parser);
			CleanupStack::PopAndDestroy(desArray);
		}
		else
		{
			HBufC*		text =	CEikonEnv::Static()->AllocReadResourceLC(aResId);
			ShowSimpleTextL(*text, aRichText);
			CleanupStack::PopAndDestroy(text);
		}
	}
}

void CSimpleTextInfoDialog::ShowSimpleTextL(const TDesC& aSimpleText, CRichText& aRichText)
{
	CSimpleTextFormatParser* parser = CSimpleTextFormatParser::NewLC();
	parser->ParseL(aSimpleText, aRichText);

	CleanupStack::PopAndDestroy(parser);
}



/************************************************
 *
 * About Dialog
 *
 ************************************************/

//CAboutDialog::CAboutDialog() : CSimpleTextInfoDialog(-1, ECtlAboutVersion, R_SIMPLE_TEXT_ABOUT_LINKS, ECtlAboutLinks)
CAboutDialog::CAboutDialog() : CSimpleTextInfoDialog(R_SIMPLE_TEXT_ABOUT, ECtlAboutVersion, -1, ECtlAboutLinks)
{
}

void CAboutDialog::ShowTextL(CRichText& aRichText, TInt aRichTextCtlId, TInt aResId)
{
	if (aRichTextCtlId == ECtlAboutLinks)
		CSimpleTextInfoDialog::ShowTextL(aRichText, aRichTextCtlId, aResId);
	else
	{
		TBuf<16>	versionText;
		TBuf<512>	text;

#if (EMAME_VERSION_PATCH > 0)
		versionText.Format(_L("%d.%d.%d"), S60CPC_VERSION_MAJOR, S60CPC_VERSION_MINOR, S60cpc_VERSION_PATCH);
#else
		versionText.Format(_L("%d.%d"), S60CPC_VERSION_MAJOR, S60CPC_VERSION_MINOR);
#endif

		HBufC*		aboutFormat =	CEikonEnv::Static()->AllocReadResourceLC(R_SIMPLE_TEXT_ABOUT);
		text.Format(*aboutFormat, &versionText);

		ShowSimpleTextL(text, aRichText);

		CleanupStack::PopAndDestroy(aboutFormat);
	}
}

// ------------------------------------------------------------------------------------------------------------------------

CAknExListColumnContainer::CAknExListColumnContainer()
:   //iLabel( NULL ),
    iListBox( NULL )
{
}

// ----------------------------------------------------------------------------
// void CAknExListColumnContainer::ConstructL( const TRect& )
// EPOC constructor.
// ----------------------------------------------------------------------------
//
void CAknExListColumnContainer::ConstructL( const TRect& aRect )
{
    CreateWindowL(); // Creates window.
	DisplayListBoxL();
    SetRect( aRect ); // Sets rectangle of frame.
    ActivateL(); // Activates window. (Ready to draw)
}

// ----------------------------------------------------------------------------
// CAknExListColumnContainer::~CAknExListColumnContainer()
// Destructor.
// ----------------------------------------------------------------------------
//
CAknExListColumnContainer::~CAknExListColumnContainer()
{
    //delete iLabel; // Deletes label object.
    delete iListBox; // Deletes listbox object.
}

// ----------------------------------------------------------------------------
// void CAknExListColumnContainer::DisplayListBoxL( const TInt )
// Displays listbox or start screen.
// ----------------------------------------------------------------------------
//
void CAknExListColumnContainer::DisplayListBoxL()
{
    if ( iListBox )
    {
        delete iListBox;
        iListBox = NULL;
    }

	iListBox = new( ELeave ) CAknSingleStyleListBox();

	if (iListBox)
	{
		//SetListBoxFromResourceL( iListBox, R_AKNEXLIST_AUDIO_SMPRATE );
		//iListBox->ConstructL(this);
		iListBox->SetContainerWindowL( *this );
		TResourceReader reader;
		iEikonEnv->CreateResourceReaderLC( reader, R_AKNEXLIST_AUDIO_SMPRATE );
		iListBox->ConstructFromResourceL( reader );
		CleanupStack::PopAndDestroy(); // resource stuffs.
		// scrollbar
		iListBox->CreateScrollBarFrameL( ETrue );
		iListBox->ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto );

		iListBox->ActivateL();

		TInt pos(iListBox->View()->CurrentItemIndex());
		if (iListBox->ScrollBarFrame())
		{
			iListBox->ScrollBarFrame()->MoveVertThumbTo(pos);
		}

		DrawNow();		
	}
}

void CAknExListColumnContainer::HandleListBoxEventL(
    CEikListBox* /*aListBox*/,
    TListBoxEvent aEventType )
{
    if ( ( aEventType == MEikListBoxObserver::EEventEnterKeyPressed ) ||
         ( aEventType == MEikListBoxObserver::EEventItemClicked ) )
    {
        // Gets current selected item index.
        TInt currentItem( iListBox->CurrentItemIndex() );
	}
}
/*
void CAknExListColumnContainer::Draw( const TRect& aRect ) const
{
    // (Re)draw the rectangle of frame.
    CWindowGc& gc = SystemGc();
    gc.SetPenStyle( CGraphicsContext::ENullPen );
    gc.SetBrushColor( TRgb(128,128,128) );
    gc.SetBrushStyle( CGraphicsContext::ESolidBrush );
    gc.DrawRect( aRect );
}
*/

void CAknExListColumnContainer::SizeChanged()
{
	//iListBox->SetExtent(TPoint(0, 0), iListBox->MinimumSize());
	//iListBox->SetExtent(TPoint(0, 0), TSize(176,144));
}