/*******************************************************************
 *
 *	File:		Dialogs.h
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __DIALOGS_H
#define __DIALOGS_H

#include <eikenv.h>
#include <eikdialg.h>
#include <eiktxlbx.h>
#include <eiktxlbm.h>
//#include <eikdlgtb.h>	// MR
#include <eiklabel.h>
//#include <eikchlst.h>
//#include <eikchkbx.h>
#include <eikedwob.h>

#include <coecntrl.h>
#include <aknsfld.h>
class CEikColumnListBox;

class CRichText;
class CEikRichTextEditor;

/************************************************
 *
 * CSimpleTextInfo Dialog
 *
 ************************************************/

class CSimpleTextInfoDialog : public CEikDialog, public MEikEdwinSizeObserver
{
public:
	CSimpleTextInfoDialog(TInt aTextIdOne = -1, TInt aRichTextCtlIdOne = -1, 
		TInt aTextIdTwo = -1, TInt aRichTextCtlIdTwo = -1,
		TBool aSimpleTextResIdOneIsArray = EFalse, TBool aSimpleTextResIdTwoIsArray = EFalse
		);
	void SetDialogBackground(TBool aEnable){iSetDialogBackground=aEnable;}
	void WantVertScrollbar(TBool aEnable){iWantVertScrollbar=aEnable;}
public:			// implements MEikEdwinSizeObserver
	virtual TBool HandleEdwinSizeEventL(CEikEdwin* aEdwin, TEdwinSizeEvent aEventType, TSize aDesirableEdwinSize);

protected: // framework
    void PreLayoutDynInitL();
	void PostLayoutDynInitL();

protected:	// new stuff
	virtual void ShowTextL(CRichText& aRichText, TInt aRichTextCtlId, TInt aResId);
	virtual void PreLayoutDynInitRichTextL(CEikRichTextEditor& aRichTextEditor, TInt aRichTextCtlId, TInt aResId);

	void ShowSimpleTextL(const TDesC& aSimpleText, CRichText& aRichText);

	TInt	iSimpleTextResIdOne;
	TInt	iSimpleTextResIdTwo;
	TInt	iRichTextCtlIdOne;
	TInt	iRichTextCtlIdTwo;
	TBool	iSimpleTextResIdOneIsArray;
	TBool	iSimpleTextResIdTwoIsArray;
	TBool	iSetDialogBackground;
	TBool	iWantVertScrollbar;
};


/************************************************
 *
 * About Dialog
 *
 ************************************************/

class CAboutDialog : public CSimpleTextInfoDialog
{
public:
	CAboutDialog();
protected:	// from CSimpleTextInfoDialog
	virtual void ShowTextL(CRichText& aRichText, TInt aRichTextCtlId, TInt aResId);
};


class CAknExListColumnContainer : public CCoeControl, public MEikListBoxObserver
{
    public: // Constructors and destructor

        /**
        * Copy constructor.
        * @param aView Pointer of CAknExListColumnView class.
        */
        CAknExListColumnContainer();

        /**
        * EPOC constructor.
        * @param aRect Frame rectangle for container.
        */
        void ConstructL( const TRect& aRect );

        /**
        * Destructor.
        */
        virtual ~CAknExListColumnContainer();

    public: // New functions

        /**
        * Displays listbox or start screen.
        * @param aOutlineId The ID of outline to displayed next.
        */
        void DisplayListBoxL();

    public: // From MEikListBoxObserver

        /**
        * Handles listbox event.
        * @param aListBox Pointer to ListBox object is not used.
        * @param aEventType Type of listbox event.
        */
        void HandleListBoxEventL(
            CEikListBox* aListBox,
            TListBoxEvent aEventType );

    private: // From CoeControl
        /**
        * From CCoeControl, Draw.
        * Fills the window's rectangle.
        * @param aRect Region of the control to be (re)drawn.
        */
        //void Draw( const TRect& aRect ) const;
		
        /**
        * From CoeControl, CountComponentControls.
        * Returns number of components.
        * @return Number of controls.
        */
		TInt CountComponentControls() const { return 1; }

        /**
        * From CoeControl, SizeChanged.
        * Called by framework when the view size is changed.
        */
        void SizeChanged();


    private: // Data
        CEikColumnListBox *iListBox;
};



#endif			/* __DIALOGS_H */