/*******************************************************************
 *
 *	File:		Engine.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "Engine.h"
#include <w32std.h>
#include <eikenv.h>
#include <eikdll.h>


#ifdef __WINS__
_LIT(KGameName, "z:\\system\\Apps\\S60cpc\\cpcemu.dll");
#else
_LIT(KGameName_C, "c:\\system\\Apps\\S60cpc\\cpcemu.exe");
_LIT(KGameName_D, "e:\\system\\Apps\\S60cpc\\cpcemu.exe");
#endif



void KillRunningStdoutConsole()
{
#ifdef __WINS__
	TInt	id = 0;
	TBool	sentEvent = EFalse;
	RWsSession&		wsSession = CEikonEnv::Static()->WsSession();

		/*
		 * When EPOC's STDLIB shuts down it will put up a "press a key to close console".
		 * In WINS, we'll find the window group belonging to the thread STDOUT and send
		 * it a key event to have it automatically closed.
		 */
	while (id >= 0)
	{
		id = wsSession.FindWindowGroupIdentifier(id , _L("*"));
		if (id > 0)
		{
			TThreadId threadId;
			if (wsSession.GetWindowGroupClientThreadId(id , threadId) == KErrNone)
			{
				RThread	thread;
				if (thread.Open(threadId) == KErrNone)
				{
					TFullName	name;
					name = thread.Name();
					if (name.Compare(_L("STDOUT")) == 0)
					{
						TWsEvent	event;
						TKeyEvent	key;
						
						key.iCode=EKeyEscape;
						key.iScanCode=0;
						key.iModifiers=0;

						event.SetType(EEventKey);
						*event.Key()=key;
						event.SetTimeNow();
						wsSession.SendEventToWindowGroup(id, event);

						sentEvent = ETrue;
					}
					thread.Close();
				}
			}
		}
	}
	if (!sentEvent)
		User::InfoPrint(_L("Please close the task named STDOUT"));
#endif
}


CGameRunner::~CGameRunner()
{
	TerminateGame(EFalse);
}

CGameRunner::CGameRunner(MGameWatcher& aGameWatcher) : CActive(CActive::EPriorityStandard), iGameWatcher(aGameWatcher)
{
}


CGameRunner* CGameRunner::NewL(MGameWatcher& aGameWatcher)
{
	CGameRunner*	self = new(ELeave) CGameRunner(aGameWatcher);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();			// self
	return self;
}

void CGameRunner::ConstructL()
{
	CActiveScheduler::Add(this);
}

void CGameRunner::RunL()
{
	iGameWatcher.NotifyGameFinished();
}

void CGameRunner::DoCancel()
{
	iThread.LogonCancel(iStatus);
}

void CGameRunner::RunGameL()
{
#ifdef __WINS__
	TThreadId threadId = EikDll::StartExeL(KGameName);
#else
	TThreadId threadId;
	TRAPD(err, threadId = EikDll::StartExeL(KGameName_C));
	if (err)
	{
		TRAP(err, threadId = EikDll::StartExeL(KGameName_D));
	}
	User::LeaveIfError(err);
#endif

	iThread.Open(threadId);
	iThread.Logon(iStatus);
	SetActive();
}


void CGameRunner::TerminateGame(TBool aNotifyObserver)
{
	Cancel();
	ShutdownGame();
	if (aNotifyObserver)
		RunL();
}

void CGameRunner::ResumeGame()
{
	RSemaphore	semaphore;
	if (semaphore.OpenGlobal(_L("S60CPC_SEM")) == KErrNone)
	{
		semaphore.Signal();
		semaphore.Close();
	}
	else
	{
		User::InfoPrint(_L("Could not resume"));
	}
}


void CGameRunner::ShutdownGame()
{
	if (iThread.Handle())
	{
#ifdef __WINS__
			/*
			 * Throw in some waits before killing it off. Thread's dying whilst running under the  
			 * may result in a emulator + devstudio freeze.
			 */
		User::After(1000 * 500);
		KillRunningStdoutConsole();
		User::After(1000 * 500);
#else
		iThread.Terminate(0);
#endif
		iThread.Close();
	}
}

