#define _CPP

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <math.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <stdarg.h>
#include <ctype.h>

#include <coedef.h>		// for ECoeWinPriorityNeverAtFront
#include <gdi.h>
#include "video.h"
#include "navigation.h"
#include "game.h"

#include  "types.h"
#include "vga.h"
#include "ppi.h"
#include "z80.h"

#ifdef USE_SOUND
#include "audio.h"
#include "polledas.h"
#endif	// USE_SOUND

#include "viddef.h"

extern "C"
{
#include "upd.h"
extern int finMain;
extern int Turbo;
extern int Sound;
}
void Quit(void);

_LIT(KFontName,"Courier");
const TInt KOneSecondInMicroSeconds =	1000 * 1000;

#ifndef __WINS__
extern "C" void DEB_Print(int i, int j, int k, int l);
#endif	// __WINS__

/*
 * TODO:
 * - orientation
 * - vk symbols
 * - vk buttons
 * - add a MGameTickObserver. Some of the MGameScreenDrawObserver should be changed to be 
 * a MGameTickObserver instead
 * - use proper optimisation options
 *		 -O3 -fstrength-reduce -fthread-jumps  -fcse-follow-jumps -fcse-skip-blocks -frerun-cse-after-loop  -fschedule-insns -fschedule-insns2 -fexpensive-optimizations  -fforce-mem -fforce-addr -funroll-loops
 */


/*****************************************
 *
 * TScaledBitmapContext implementation
 *
 *****************************************/

TScaledBitmapContext::TScaledBitmapContext() : iSkipX(-1), iSkipY(-1), iSrcGameScreenDevice(NULL)
{
}


TScaledBitmapContext::TScaledBitmapContext(const CGameScreenDevice& aDstGameScreenDevice, const CGameScreenDevice& aSrcGameScreenDevice, const TSize& aBitmapSize, TBool aCenter)
	: iSrcGameScreenDevice(&aSrcGameScreenDevice)
{
	float	scaleX = ((float) aDstGameScreenDevice.Size().iWidth) / ((float) aBitmapSize.iWidth);
	float	scaleY = ((float) aDstGameScreenDevice.Size().iHeight) / ((float) aBitmapSize.iHeight);
	if ((scaleX < 1) || (scaleY < 1))
	{
		float scale = (scaleX < scaleY) ? scaleX : scaleY;

		// ideal size
		iSize.iWidth = (TInt) (scale * aBitmapSize.iWidth);
		iSize.iHeight = (TInt) (scale * aBitmapSize.iHeight);

		iSkipX = (TInt) floor (((float) aBitmapSize.iWidth) / (((float) aBitmapSize.iWidth) - ((float)iSize.iWidth)));
		iSkipY = (TInt) floor (((float) aBitmapSize.iHeight) / (((float) aBitmapSize.iHeight) - ((float) iSize.iHeight)));

		// real size based on the pixels we are going to skip
		iSize.iWidth = aBitmapSize.iWidth - (aBitmapSize.iWidth / iSkipX);
		iSize.iHeight = aBitmapSize.iHeight - (aBitmapSize.iHeight / iSkipY);

		iSkipY--;	// we start counting at zero
		iSkipX--;
	}

	if (aCenter)
	{
		iPoint.iX = (aDstGameScreenDevice.Size().iWidth - iSize.iWidth) / 2;
		iPoint.iY = (aDstGameScreenDevice.Size().iHeight - iSize.iHeight) / 2;
	}
	iScanLineBytes = iSize.iWidth * (TGameBitmapUtil::GetBpp(aDstGameScreenDevice.DisplayMode())/8);
	ASSERT(iScanLineBytes < (1024 * 4));
}



/*****************************************
 *
 * TGameBitmapUtil implementation
 *
 *****************************************/

TRect TGameBitmapUtil::VisibleIntersection(const TSize& aScreenSize, const TPoint& aOrigin, const TSize& aBitmapSize, const TRect& aBitmapRect)
{
	TRect	screenRect = TRect(aScreenSize);
	TRect	bitmapRect = TRect(aBitmapSize);
	bitmapRect.Intersection(aBitmapRect);
	bitmapRect.Move(aOrigin);
	screenRect.Intersection(bitmapRect);
	return screenRect;
}


TUint8* TGameBitmapUtil::PixelAddress(const CGameScreenDevice* aSrc, const TPoint& aPoint)
{
	return aSrc->ScanLineAddress(aPoint.iY) + ((aPoint.iX * GetBpp(aSrc->DisplayMode())) / 8);
}

TDisplayMode TGameBitmapUtil::GetDisplayMode(TInt aBpp, TBool aColor)
{
	if (aBpp == 1)
		return EGray2;
	else if (aBpp == 2)
		return EGray4;
	else if (aBpp == 4)
		return aColor ? EColor16 : EGray16;
	else if (aBpp == 8)
		return aColor ? EColor256 : EGray256;
	else if (aBpp == 16)
		return EColor64K;
	else if (aBpp == 12)
		return EColor4K;
	else if (aBpp == 24)
		return EColor16M;
	else if (aBpp == 32)
		return ERgb;

	ASSERT(EFalse);
	return ENone;
}

/*
 * GetBpp returns the number of bits we need per pixel in the given
 * display mode. It is different from GetColorDepth in that GetBpp can be
 * used to calculate byte offsets in a bitmap/framebuffer.
 * For example, GetBpp(EColor4K) returns 16 because each pixel requires
 * 16 bits (of which only the least significant 12 bits are used).
 * GetColorDepth(EColor4K) returns 12.
 */
TInt TGameBitmapUtil::GetBpp(TDisplayMode aDisplayMode)
{
	const TInt bpp[] =
		{
			-1,
			1,
			2,
			4,
			8,
			4,
			8,
			16,
			32,
			32,
			16
		};
	return bpp[aDisplayMode];
}

TInt TGameBitmapUtil::GetColorDepth(TDisplayMode aDisplayMode)
{
	const TInt bpp[] =
		{
			-1,
			1,
			2,
			4,
			8,
			4,
			8,
			16,
			24,
			32,
			12
		};
	return bpp[aDisplayMode];
}

TUint TGameBitmapUtil::GetColorValue(const TRgb& aColor, const TDisplayMode& aDisplayMode)
{
	switch (aDisplayMode)
	{
		case EColor256:
			return aColor.Color256();
			break;
		case EColor4K:
			return aColor.Color4K();
			break;
		case EColor16M:
			return aColor.Color16M();
			break;
		case EColor64K:
			return aColor.Color64K();
			break;
		case ERgb:
			return aColor.Value();
			break;
		default:
			ASSERT(EFalse);
	}
	return 0;
}


/*****************************************
 *
 * CGamePalette implementation
 *
 *****************************************/

CGamePalette::CGamePalette(TDisplayMode aDisplayMode) : iDisplayMode(aDisplayMode)
{
}

CGamePalette::~CGamePalette()
{
}

CGamePalette* CGamePalette::NewL(TDisplayMode aDisplayMode)
{
	CGamePalette*	self = new(ELeave)CGamePalette(aDisplayMode);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}


void CGamePalette::ConstructL()
{
	iPaletteMaxIndex = 256;
	Mem::FillZ(iPalette, sizeof(iPalette));
	iPaletteLastIndexUsed = 0;
}



TInt CGamePalette::AllocateEntry(const TRgb& aColor)
{
	TUint	colorValue = TGameBitmapUtil::GetColorValue(aColor, iDisplayMode);
	for (TInt i=0 ; i<iPaletteLastIndexUsed; i++)
		if (iPalette[i] == colorValue)
			return i;
	ASSERT(iPaletteLastIndexUsed<iPaletteMaxIndex);
	iPalette[iPaletteLastIndexUsed] = colorValue;
	return (iPaletteLastIndexUsed++);
}



/*****************************************
 *
 * CGameScreenDevice implementation
 *
 *****************************************/

void CGameScreenDevice::DirectDraw_16bpp_16bppRotLeft(const TPoint& aDstOrigin, const CGameScreenDevice& aSrcDevice, const TPoint& aSrcOrigin, const TSize& aSize)
{
	TInt					width = aSize.iWidth;
	TInt					height = aSize.iHeight;

	register TUint16*		dstLine = (TUint16*) TGameBitmapUtil::PixelAddress(this, aDstOrigin);
	register TUint16*		srcLine = (TUint16*) TGameBitmapUtil::PixelAddress(&aSrcDevice, aSrcOrigin);

	for(int i=0 ; i<height ; i++)
	{
		TUint16*			dst = dstLine;
		TUint16*			src = srcLine;
		for(int j=0 ; j<width ; j++)
		{
			*dst = *src++;
			dst += BytesPerLine()>>1;
		}
		dstLine--;
		srcLine += aSrcDevice.BytesPerLine()>>1;
	}
}

void CGameScreenDevice::DirectDraw_16bpp_16bppRotRight(const TPoint& aDstOrigin, const CGameScreenDevice& aSrcDevice, const TPoint& aSrcOrigin, const TSize& aSize)
{
	TInt					width = aSize.iWidth;
	TInt					height = aSize.iHeight;

	register TUint16*		dstLine = (TUint16*) TGameBitmapUtil::PixelAddress(this, aDstOrigin);
	register TUint16*		srcLine = (TUint16*) TGameBitmapUtil::PixelAddress(&aSrcDevice, aSrcOrigin);

	for(int i=0 ; i<height ; i++)
	{
		TUint16*			dst = dstLine;
		TUint16*			src = srcLine;
		for(int j=0 ; j<width ; j++)
		{
			*dst = *src++;
			dst -= BytesPerLine()>>1;
		}
		dstLine++;
		srcLine += aSrcDevice.BytesPerLine()>>1;
	}
}


CGameScreenDevice::CGameScreenDevice(const TSize& aSize, TDisplayMode aDisplayMode) : iSize(aSize), iDisplayMode(aDisplayMode)
{
}

CGameScreenDevice::~CGameScreenDevice()
{
}


const CGameScreenDevice::TDirectDrawer CGameScreenDevice::KDirectDrawers12or16bpp[KVideoModes] = 
{
	&CGameScreenDevice::DirectDraw_Unsupported,			/* ENone */
	&CGameScreenDevice::DirectDraw_Unsupported,			/* EGray2 */
	&CGameScreenDevice::DirectDraw_Unsupported,			/* EGray4 */
	&CGameScreenDevice::DirectDraw_Unsupported,			/* EGray16 */
	&CGameScreenDevice::DirectDraw_12or16bpp_8bpp,		/* EGray256 */
	&CGameScreenDevice::DirectDraw_Unsupported,			/* EColor16 */
	&CGameScreenDevice::DirectDraw_12or16bpp_8bpp,		/* EColor256 */
	&CGameScreenDevice::DirectDraw_12or16bpp_12or16bpp,	/* EColor64K */
	&CGameScreenDevice::DirectDraw_Unsupported,			/* EColor16M */
	&CGameScreenDevice::DirectDraw_Unsupported,			/* ERgb */
	&CGameScreenDevice::DirectDraw_12or16bpp_12or16bpp	/* EColor4K */
};

const CGameScreenDevice::TDirectDrawer CGameScreenDevice::KDirectDrawers8bpp[KVideoModes] = 
{
	&CGameScreenDevice::DirectDraw_Unsupported,	/* ENone */
	&CGameScreenDevice::DirectDraw_Unsupported,	/* EGray2 */
	&CGameScreenDevice::DirectDraw_Unsupported,	/* EGray4 */
	&CGameScreenDevice::DirectDraw_Unsupported,	/* EGray16 */
	&CGameScreenDevice::DirectDraw_8bpp_8bpp,	/* EGray256 */
	&CGameScreenDevice::DirectDraw_Unsupported,	/* EColor16 */
	&CGameScreenDevice::DirectDraw_8bpp_8bpp,	/* EColor256 */
	&CGameScreenDevice::DirectDraw_Unsupported,	/* EColor64K */
	&CGameScreenDevice::DirectDraw_Unsupported,	/* EColor16M */
	&CGameScreenDevice::DirectDraw_Unsupported,	/* ERgb */
	&CGameScreenDevice::DirectDraw_Unsupported	/* EColor4K */
};


void CGameScreenDevice::Clear()
{
	TInt	height = iSize.iHeight;
	for (TInt h=0 ; h<height ; h++)
	{
		TUint8*		line = ScanLineAddress(h);
		Mem::FillZ(line, BytesPerLine());
	}
}


void CGameScreenDevice::DirectDraw(const TPoint& aPoint, const CGameScreenDevice& aSrcDevice, const TRect& aRect)
{
	TRect	rect = TGameBitmapUtil::VisibleIntersection(Size(), aPoint, aSrcDevice.Size(), aRect);
	TPoint	dstPoint = aRect.iTl;
	if (aPoint.iX < 0)
		dstPoint.iX -= aPoint.iX;
	if (aPoint.iY < 0)
		dstPoint.iY -= aPoint.iY;

	switch (DisplayMode())
	{
		case EColor256:
		case EGray256:
			(this->*KDirectDrawers8bpp[aSrcDevice.DisplayMode()])(rect.iTl, aSrcDevice, dstPoint, rect.Size());
			break;
		case EColor4K:
		case EColor64K:
			(this->*KDirectDrawers12or16bpp[aSrcDevice.DisplayMode()])(rect.iTl, aSrcDevice, dstPoint, rect.Size());
			break;
		default:
			ASSERT(EFalse);
			break;
	}
}



const CGameScreenDevice::TAlphaBlender CGameScreenDevice::KAlphaBlenders12bpp[KVideoModes] = 
{
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* ENone */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* EGray2 */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* EGray4 */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* EGray16 */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* EGray256 */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* EColor16 */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* EColor256 */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* EColor64K */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* EColor16M */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* ERgb */
	&CGameScreenDevice::AlphaBlend_12bpp_12bpp	/* EColor4K */
};

const CGameScreenDevice::TAlphaBlender CGameScreenDevice::KAlphaBlenders16bpp[KVideoModes] = 
{
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* ENone */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* EGray2 */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* EGray4 */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* EGray16 */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* EGray256 */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* EColor16 */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* EColor256 */
	&CGameScreenDevice::AlphaBlend_16bpp_16bpp,	/* EColor64K */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* EColor16M */
	&CGameScreenDevice::AlphaBlend_Unsupported,	/* ERgb */
	&CGameScreenDevice::AlphaBlend_Unsupported	/* EColor4K */
};

void CGameScreenDevice::AlphaBlend(const TPoint& aPoint, const CGameScreenDevice& aSrcDevice, const TRect& aRect, TInt8 aAlpha)
{
	TRect	rect = TGameBitmapUtil::VisibleIntersection(Size(), aPoint, aSrcDevice.Size(), aRect);
	TPoint	dstPoint = aRect.iTl;
	if (aPoint.iX < 0)
		dstPoint.iX -= aPoint.iX;
	if (aPoint.iY < 0)
		dstPoint.iY -= aPoint.iY;

	switch (DisplayMode())
	{
		case EColor4K:
			(this->*KAlphaBlenders12bpp[aSrcDevice.DisplayMode()])(rect.iTl, aSrcDevice, dstPoint, rect.Size(), aAlpha);
			break;
		case EColor64K:
			(this->*KAlphaBlenders16bpp[aSrcDevice.DisplayMode()])(rect.iTl, aSrcDevice, dstPoint, rect.Size(), aAlpha);
			break;
		default:
			ASSERT(EFalse);
			break;
	}
}


void CGameScreenDevice::DirectDoubleDraw(const TPoint& /*aPoint*/, const CGameScreenDevice& /*aSrcDevice*/, const TRect& /*aRect*/)
{
#if 0
	TRect	rect = TGameBitmapUtil::VisibleIntersection(Size(), aPoint, aSrcDevice.Size(), aRect);
	TPoint	dstPoint = aRect.iTl;
	if (aPoint.iX < 0)
		dstPoint.iX -= aPoint.iX;
	if (aPoint.iY < 0)
		dstPoint.iY -= aPoint.iY;
#endif
}


const CGameScreenDevice::TScaledScanLineDrawer CGameScreenDevice::KScaledScanLineDrawers12or16bpp[KVideoModes] = 
{
	&CGameScreenDevice::ScaledScanLineDraw_Unsupported,		/* ENone */
	&CGameScreenDevice::ScaledScanLineDraw_Unsupported,		/* EGray2 */
	&CGameScreenDevice::ScaledScanLineDraw_Unsupported,		/* EGray4 */
	&CGameScreenDevice::ScaledScanLineDraw_Unsupported,		/* EGray16 */
	&CGameScreenDevice::ScaledScanLineDraw_12or16bpp_8bpp,	/* EGray256 */
	&CGameScreenDevice::ScaledScanLineDraw_Unsupported,		/* EColor16 */
	&CGameScreenDevice::ScaledScanLineDraw_12or16bpp_8bpp,	/* EColor256 */
	&CGameScreenDevice::ScaledScanLineDraw_Unsupported,		/* EColor64K */
	&CGameScreenDevice::ScaledScanLineDraw_Unsupported,		/* EColor16M */
	&CGameScreenDevice::ScaledScanLineDraw_Unsupported,		/* ERgb */
	&CGameScreenDevice::ScaledScanLineDraw_Unsupported		/* EColor4K */
};

void my_memcpy(void* out, const void* in, size_t length)
{
	TUint16*	dst = (TUint16*) out;
	TUint16*	src = (TUint16*) in;

	length>>=1;

	while (length--)
	{
		*dst++ = *src++;
	}
}

void CGameScreenDevice::ScaledDrawBitmap(const TScaledBitmapContext& aScaledBitmapContext)
{
	CGameScreenDevice::TScaledScanLineDrawer scaledScanLineDrawer= NULL;

	switch (DisplayMode())
	{
		case EColor4K:
		case EColor64K:
			scaledScanLineDrawer = KScaledScanLineDrawers12or16bpp[aScaledBitmapContext.SrcGameScreenDevice().DisplayMode()];
			break;
		default:
			break;
	}
	if (!scaledScanLineDrawer)
	{
		ASSERT(EFalse);
		return;
	}


	TUint8*			dst_line = (TUint8*) TGameBitmapUtil::PixelAddress(this, aScaledBitmapContext.Point());
	const CGameScreenDevice& srcGameScreenDevice = aScaledBitmapContext.SrcGameScreenDevice();

	TInt	height = aScaledBitmapContext.SrcGameScreenDevice().Size().iHeight;
	TInt	lineCount = 0;
	TInt	temp=0;

	for (TInt h=0 ; h<height ; h++)
	{
		if (lineCount == aScaledBitmapContext.SkipY())
		{
			(this->*scaledScanLineDrawer)(aScaledBitmapContext, (TUint*)srcGameScreenDevice.ScanLineAddress(h), ETrue);
			lineCount = 0;
		}
		else
		{
			(this->*scaledScanLineDrawer)(aScaledBitmapContext, (TUint*) srcGameScreenDevice.ScanLineAddress(h), EFalse);
			lineCount++;
		}
		if (lineCount != aScaledBitmapContext.SkipY())
		{
			my_memcpy(dst_line, aScaledBitmapContext.ScanLine(), aScaledBitmapContext.ScanLineBytes());
			dst_line += BytesPerLine();
			temp++;
		}
	}
}


void CGameScreenDevice::DirectDraw_12or16bpp_8bpp(const TPoint& aDstOrigin, const CGameScreenDevice& aSrcDevice, const TPoint& aSrcOrigin, const TSize& aSize)
{
	ASSERT(aSrcDevice.Palette());
	ASSERT(iDisplayMode == aSrcDevice.Palette()->DisplayMode());

	TUint*			pal = aSrcDevice.Palette()->PaletteBuffer();
	TUint16*		dst_line = (TUint16*) TGameBitmapUtil::PixelAddress(this, aDstOrigin);
	const TUint8*	src_line = TGameBitmapUtil::PixelAddress(&aSrcDevice, aSrcOrigin);

	if (!pal)
		return;

	TInt	height = aSize.iHeight;
	TInt	width = aSize.iWidth;

	for(TInt h=0 ; h<height ; h++)
	{
		TUint16*		dst = dst_line;
		const TUint8*	src = src_line;
		for (TInt w=0 ; w<width ; w++)
		{
			*dst++ = (TUint16) pal[*src++];
		}
		dst_line += (BytesPerLine() >> 1);
		src_line += aSrcDevice.BytesPerLine();
	}
}


void CGameScreenDevice::DirectDraw_8bpp_8bpp(const TPoint& aDstOrigin, const CGameScreenDevice& aSrcDevice, const TPoint& aSrcOrigin, const TSize& aSize)
{
	ASSERT(aSrcDevice.Palette());
	ASSERT(iDisplayMode == aSrcDevice.Palette()->DisplayMode());

	TUint8*			dst_line = (TUint8*) TGameBitmapUtil::PixelAddress(this, aDstOrigin);
	const TUint8*	src_line = TGameBitmapUtil::PixelAddress(&aSrcDevice, aSrcOrigin);

	TInt	height = aSize.iHeight;
	TInt	width = aSize.iWidth;

	for(TInt h=0 ; h<height ; h++)
	{
		Mem::Copy(dst_line, src_line, width);
		dst_line += (BytesPerLine() >> 1);
		src_line += aSrcDevice.BytesPerLine();
	}
}




void CGameScreenDevice::DirectDraw_12or16bpp_12or16bpp(const TPoint& aDstOrigin, const CGameScreenDevice& aSrcDevice, const TPoint& aSrcOrigin, const TSize& aSize)
{
	TUint16*		dst_line = (TUint16*) TGameBitmapUtil::PixelAddress(this, aDstOrigin);
	const TUint16*	src_line = (TUint16*) TGameBitmapUtil::PixelAddress(&aSrcDevice, aSrcOrigin);

	TInt	height = aSize.iHeight;
	TInt	width = aSize.iWidth;

	for(TInt h=0 ; h<height ; h++)
	{
		TUint16*		dst = dst_line;
		const TUint16*	src = src_line;
		for (TInt w=0 ; w<width ; w++)
		{
			*dst++ = *src++;
		}
		dst_line += (BytesPerLine() >> 1);
		src_line += (aSrcDevice.BytesPerLine() >> 1);
	}
}


void CGameScreenDevice::AlphaBlend_16bpp_16bpp(const TPoint& aDstOrigin, const CGameScreenDevice& aSrcDevice, const TPoint& aSrcOrigin, const TSize& aSize, TUint8 aAlpha)
{
	TUint16*		dst_line = (TUint16*) TGameBitmapUtil::PixelAddress(this, aDstOrigin);
	const TUint16*	src_line = (TUint16*) TGameBitmapUtil::PixelAddress(&aSrcDevice, aSrcOrigin);

	TInt	height = aSize.iHeight;
	TInt	width = aSize.iWidth;

	for(TInt h=0 ; h<height ; h++)
	{
		TUint16*		dst = dst_line;
		const TUint16*	src = src_line;
		for (TInt w=0 ; w<width ; w++)
		{
			// alpha blending algorithm "borrowed" from MicroWindows ;-)
			unsigned int s = *src++;
			unsigned int d = *dst;
			unsigned int t = d & 0xf800;
			unsigned int m1, m2, m3;
			m1 = (((((s & 0xf800) - t)*aAlpha)>>8) & 0xf800) + t;
			t = d & 0x07e0;
			m2 = (((((s & 0x07e0) - t)*aAlpha)>>8) & 0x07e0) + t;
			t = d & 0x001f;
			m3 = (((((s & 0x001f) - t)*aAlpha)>>8) & 0x001f) + t;
			*dst++ = (TUint16) (m1 | m2 | m3);
		}
		dst_line += (BytesPerLine() >> 1);
		src_line += (aSrcDevice.BytesPerLine() >> 1);
	}
}

void CGameScreenDevice::AlphaBlend_12bpp_12bpp(const TPoint& aDstOrigin, const CGameScreenDevice& aSrcDevice, const TPoint& aSrcOrigin, const TSize& aSize, TUint8 aAlpha)
{
	TUint16*		dst_line = (TUint16*) TGameBitmapUtil::PixelAddress(this, aDstOrigin);
	const TUint16*	src_line = (TUint16*) TGameBitmapUtil::PixelAddress(&aSrcDevice, aSrcOrigin);

	TInt	height = aSize.iHeight;
	TInt	width = aSize.iWidth;

	for(TInt h=0 ; h<height ; h++)
	{
		TUint16*		dst = dst_line;
		const TUint16*	src = src_line;
		for (TInt w=0 ; w<width ; w++)
		{
			// alpha blending algorithm "borrowed" from MicroWindows ;-)
			unsigned int s = *src++;
			unsigned int d = *dst;
			unsigned int t = d & 0xf00;
			unsigned int m1, m2, m3;
			m1 = (((((s & 0xf00) - t)*aAlpha)>>8) & 0xf00) + t;
			t = d & 0x0f0;
			m2 = (((((s & 0xf0) - t)*aAlpha)>>8) & 0xf0) + t;
			t = d & 0xf;
			m3 = (((((s & 0xf) - t)*aAlpha)>>8) & 0xf) + t;
			*dst++ = (TUint16) (m1 | m2 | m3);
		}
		dst_line += (BytesPerLine() >> 1);
		src_line += (aSrcDevice.BytesPerLine() >> 1);
	}
}



void CGameScreenDevice::ScaledScanLineDraw_12or16bpp_8bpp(const TScaledBitmapContext& aScaledBitmapContext, TUint* aSrcLine, TBool aMerge)
{
	TInt		pixelCount = 0;
	TUint8*		src = (TUint8*) aSrcLine;
	TUint16*	dst = (TUint16*) aScaledBitmapContext.ScanLine();
	TUint*		pal = aScaledBitmapContext.SrcGameScreenDevice().Palette()->PaletteBuffer();
	TInt		width = aScaledBitmapContext.SrcGameScreenDevice().Size().iWidth;

	for (TInt w=0 ; w<width ; w++)
	{
		if (pixelCount == aScaledBitmapContext.SkipX())
		{
			pixelCount = 0;
			*(dst -1) |= pal[*src];
		}
		else
		{
			if (aMerge)
			{
				*dst++ |= pal[*src];
			}
			else
			{
				*dst++ = (TInt16) pal[*src];
			}
			pixelCount++;
		}
		src++;
	}
}



/*****************************************
 *
 * CFbsGameScreenDevice implementation
 *
 *****************************************/

CFbsGameScreenDevice::CFbsGameScreenDevice(const TSize& aSize, TDisplayMode aDisplayMode) : CGameScreenDevice(aSize, aDisplayMode)
{
	iBytesPerLine = ((TGameBitmapUtil::GetBpp(aDisplayMode) * aSize.iWidth ) + 7) / 8;
}

CFbsGameScreenDevice::~CFbsGameScreenDevice()
{
	delete iBitmapContext;
	iBitmapContext = NULL;
	delete iBitmapDevice;
	iBitmapDevice = NULL;
	delete iBitmap;
	iBitmap = NULL;
}


CFbsGameScreenDevice* CFbsGameScreenDevice::NewL(const TSize& aSize, TDisplayMode aDisplayMode)
{
	CFbsGameScreenDevice*	self = new(ELeave) CFbsGameScreenDevice(aSize, aDisplayMode);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}

TUint8* CFbsGameScreenDevice::ScanLineAddress(TInt aLine) const
{
//	return (TUint8 *) iBitwiseBitmap->ScanLineAddress(iDataAddress, aLine);
	return (TUint8 *) iBitwiseBitmap->ScanLineAddress(iBitmap->DataAddress(), aLine);
}


void CFbsGameScreenDevice::ConstructL()
{
	iBitmap = new (ELeave) CExtraFbsBitmap;
	iBitmap->Create(iSize, iDisplayMode);

	iBitmapDevice = CFbsBitmapDevice::NewL(iBitmap);
	iBitmapDevice->CreateContext(iBitmapContext);

	iDataAddress = iBitmap->DataAddress();
	iBitwiseBitmap =iBitmap->BitmapAdressAddress();
}





/*****************************************
 *
 * CHwFbGameScreenDevice implementation
 *
 *****************************************/

CHwFbGameScreenDevice* CHwFbGameScreenDevice::NewL(const TSize& aWindowSize, TDisplayMode aDisplayMode)
{
	TScreenInfoV01			screenInfo;
	TPckg<TScreenInfoV01>	sI(screenInfo);
	UserSvr::ScreenInfo(sI);

	if (!screenInfo.iScreenAddressValid)
		User::Leave(KErrNotSupported);

		/*
		 * Note that aWindowSize may differ from screenInfo.iScreenSize.
		 * screenInfo.iScreenSize reflects the video controller's configured pixel dimensions, 
		 * whereas aWindowSize is the LCD's screen's maximim dimension.
		 */

	//DEB_Print(aWindowSize.iWidth, aWindowSize.iHeight, screenInfo.iScreenSize.iWidth, screenInfo.iScreenSize.iHeight);

	CHwFbGameScreenDevice*	self = new(ELeave) CHwFbGameScreenDevice(aWindowSize, aDisplayMode);
	CleanupStack::PushL(self);
	self->ConstructL(screenInfo);
	CleanupStack::Pop();
	return self;
}


CHwFbGameScreenDevice::CHwFbGameScreenDevice(const TSize& aWindowSize, TDisplayMode aDisplayMode) : CGameScreenDevice(aWindowSize, aDisplayMode)
{
}

void CHwFbGameScreenDevice::ConstructL(const TScreenInfoV01& aScreenInfo)
{
	iFrameBuffer = (TUint8*) aScreenInfo.iScreenAddress;
	iBytesPerLine = ((TGameBitmapUtil::GetBpp(iDisplayMode) * aScreenInfo.iScreenSize.iWidth ) + 7) / 8;

	//DEB_Print((int)iFrameBuffer, iBytesPerLine, 0, 0);

//	RDebug::Print(_L("framebuffer=0x%x"), iFrameBuffer);

#ifdef __ER5__
		/*
		 * Hmm, I've just checked out the SA1100 spec. It seems that for 8bpp displays
		 * there is a 512 palette table at the beginning of the frame buffer.
		 * This code assumes that all ER5 target build with 8bpp are SA1100 machines. 
		 * I only know 2: Series 7 and Netbook.
		 */
	if (iDisplayMode == EColor8)
		iFrameBuffer += 512;
#endif
}


TUint8* CHwFbGameScreenDevice::ScanLineAddress(TInt aLine) const
{
	return iFrameBuffer + (BytesPerLine() * aLine);
}


/*****************************************
 *
 * CMemGameScreenDevice implementation
 *
 *****************************************/

CMemGameScreenDevice* CMemGameScreenDevice::NewL(const TSize& aSize, TDisplayMode aDisplayMode, TUint8* aMemBuffer)
{
	CMemGameScreenDevice*	self = new(ELeave) CMemGameScreenDevice(aSize, aDisplayMode, aMemBuffer);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}


CMemGameScreenDevice::CMemGameScreenDevice(const TSize& aSize, TDisplayMode aDisplayMode, TUint8* aMemBuffer) : CGameScreenDevice(aSize, aDisplayMode), iMemBuffer(aMemBuffer)
{
	iOwnsMemBuf = (iMemBuffer == NULL);
	iBytesPerLine = ((TGameBitmapUtil::GetBpp(aDisplayMode) * aSize.iWidth ) + 7) / 8;
}


CMemGameScreenDevice::~CMemGameScreenDevice()
{
	if (iOwnsMemBuf)
	{
		User::Free(iMemBuffer);
		iMemBuffer = NULL;
	}
}

void CMemGameScreenDevice::ConstructL()
{
	if (iOwnsMemBuf)
	{
		TInt	size = iBytesPerLine*iSize.iHeight;
		iMemBuffer = (TUint8*) User::AllocL(size);
		Mem::FillZ(iMemBuffer, size);
	}
}


TUint8* CMemGameScreenDevice::ScanLineAddress(TInt aLine) const
{
	return iMemBuffer + (BytesPerLine() * aLine);
}

//const TInt KVirtualKeyboardHotkey = 0x31;
const TInt KVirtualKeyboardHotkey = 0x00;

/***********************************************
 *
 * CVirtualKeyboard implementation
 *
 ***********************************************/


const TInt KVkRows = 8;
const TInt KVkCols = 12;

const TInt KKeyCodeSym		= 2;
const TInt KKeyCodeAlphaInc	= 4;
const TInt KKeyCodeAlphaDec	= 5;

const TInt KVkXMargin	= 1;
const TInt KVkYMargin	= 1;
const TInt KVkStandardWidth		= 16;
const TInt KVkStandardHeight	= 16;

#define IS_VALID_VIRTUAL_KEY(vk) ((vk)->iWidth)

#define KEY_ENTER		0x01
#define KEY_CAPSLOCK	0x02
#define KEY_SHIFT		0x03
#define KEY_ESC			0x04
#define KEY_CTRL		0x05
#define KEY_JOYL		0x06
#define KEY_JOYR		0x07
#define KEY_JOYU		0x08
#define KEY_JOYD		0x09
#define KEY_JOYF1		0x0a
#define KEY_JOYF2		0x0b
#define KEY_BACKS		0x0c
#define KEY_TAB			0x0d
#define KEY_UP			0x0e
#define KEY_DOWN		0x0f
#define KEY_LEFT		0x10
#define KEY_RIGHT		0x11
#define KEY_COPY		0x12
#define KEY_P1			0x13
#define KEY_P2			0x14
#define KEY_P3			0x15
#define KEY_P4			0x16
#define KEY_P5			0x17
#define KEY_P6			0x18
#define KEY_P7			0x19
#define KEY_P8			0x1a
#define KEY_P9			0x1b
#define KEY_P0			0x1c
#define KEY_PDOT		0x1d

const TVirtualKey	iDefaultVirtualKeyboardTable[KVkRows][KVkCols] = 
{
	{	/*                                         SHIFT  
		  width           scan    label  key    label  key
                          code          code          code */
		{KVkStandardWidth,   2,     "1", '1',     "!", '!'},
		{KVkStandardWidth,   3,     "2", '2',     "\"", '"'},
		{KVkStandardWidth,   4,     "3", '3',     "#", '#'},
		{KVkStandardWidth,   5,     "4", '4',     "$", '$'},
		{KVkStandardWidth,   6,     "5", '5',     "%", '%'},
		{KVkStandardWidth,   7,     "6", '6',     "&", '^'},
		{KVkStandardWidth,   8,     "7", '7',     "\'", '\''},
		{KVkStandardWidth,   9,     "8", '8',     "(", '('},
		{KVkStandardWidth,  10,     "9", '9',     ")", ')'},
		{KVkStandardWidth,  11,     "0", '0',     "_", '_'},
		{KVkStandardWidth + (KVkStandardWidth/2),   EStdKeyBackspace,     "<-", KEY_BACKS,     "<-", KEY_BACKS}
	},
	{
		{KVkStandardWidth + (KVkStandardWidth/2),  EStdKeyTab,     "tab", KEY_TAB,     "TAB", KEY_TAB},
		{KVkStandardWidth,  16,     "q", 'q',     "Q", 'Q'},
		{KVkStandardWidth,  17,     "w", 'w',     "W", 'W'},
		{KVkStandardWidth,  18,     "e", 'e',     "E", 'E'},
		{KVkStandardWidth,  19,     "r", 'r',     "R", 'R'},
		{KVkStandardWidth,  20,     "t", 't',     "T", 'T'},
		{KVkStandardWidth,  21,     "y", 'y',     "Y", 'Y'},
		{KVkStandardWidth,  22,     "u", 'u',     "U", 'U'},
		{KVkStandardWidth,  23,     "i", 'i',     "I", 'I'},
		{KVkStandardWidth,  24,     "o", 'o',     "O", 'O'},
		{KVkStandardWidth,  25,     "p", 'p',     "P", 'P'}
	},
	{
		//{KVkStandardWidth + (KVkStandardWidth/2),  EStdKeyCapsLock,     "cps", EKeyCapsLock,     "CPS", EKeyCapsLock},
		{KVkStandardWidth + (KVkStandardWidth/2),  EStdKeyCapsLock,     "cps", KEY_CAPSLOCK,     "CPS", KEY_CAPSLOCK},
		{KVkStandardWidth,  30,     "a", 'a',     "A", 'A'},
		{KVkStandardWidth,  31,     "s", 's',     "S", 'S'},
		{KVkStandardWidth,  32,     "d", 'd',     "D", 'D'},
		{KVkStandardWidth,  33,     "f", 'f',     "F", 'F'},
		{KVkStandardWidth,  34,     "g", 'g',     "G", 'G'},
		{KVkStandardWidth,  35,     "h", 'h',     "H", 'H'},
		{KVkStandardWidth,  36,     "j", 'j',     "J", 'J'},
		{KVkStandardWidth,  37,     "k", 'k',     "K", 'K'},
		{KVkStandardWidth,  38,     "l", 'l',     "L", 'L'},
		{KVkStandardWidth,  59,     "@", '@',     "|", '|'}		// 59 == scan code F1
	},
	{
		{KVkStandardWidth + (KVkStandardWidth/2),  EStdKeyLeftShift,     "sft", KEY_SHIFT,     "SFT", KEY_SHIFT},
		{KVkStandardWidth,  44,     "z", 'z',     "Z", 'Z'},
		{KVkStandardWidth,  45,     "x", 'x',     "X", 'X'},
		{KVkStandardWidth,  46,     "c", 'c',     "C", 'C'},
		{KVkStandardWidth,  47,     "v", 'v',     "V", 'V'},
		{KVkStandardWidth,  48,     "b", 'b',     "B", 'B'},
		{KVkStandardWidth,  49,     "n", 'n',     "N", 'N'},
		{KVkStandardWidth,  50,     "m", 'm',     "M", 'M'},
		//{(KVkStandardWidth * 3) + (KVkXMargin*2), DEVICE_SELECT_SCANCODE,     "enter", DEVICE_SELECT_KEYCODE,     "ENTER", DEVICE_SELECT_KEYCODE},
		{(KVkStandardWidth * 3) + (KVkXMargin*2), DEVICE_SELECT_SCANCODE,     "enter", KEY_ENTER,     "ENTER", KEY_ENTER},
		{0, 0,     NULL, 0,     NULL, 0},
		{0, 0,     NULL, 0,     NULL, 0}
	},
	{
		{KVkStandardWidth + (KVkStandardWidth/2),  DEVICE_ESC_SCANCODE,     "esc", KEY_ESC,     "ESC", KEY_ESC},
		{KVkStandardWidth,	0,		",", ',',     "<", '<'},
		{KVkStandardWidth,	0,		".", '.',     ">", '>'},
		{KVkStandardWidth,	0,		":", ':',     "*", '*'},
		{(KVkStandardWidth * 3) + (KVkXMargin*2), EStdKeySpace,     "space", ' ',     "space", ' '},
		{KVkStandardWidth,	0,		";", ';',     "+", '+'},
		{KVkStandardWidth,	0,		"/", '/',     "?", '?'},
		{KVkStandardWidth,	0,		"\\", '\\',     "\\", '\\'},
		{KVkStandardWidth,	0,		"^", '^',     "", ''},
		{0, 0,     NULL, 0,     NULL, 0},
		{0, 0,     NULL, 0,     NULL, 0}
	},
	{
		{KVkStandardWidth + (KVkStandardWidth/2),  EStdKeyLeftShift,     "ctrl", KEY_CTRL,     "ctrl", KEY_CTRL},
		{KVkStandardWidth,  0,     "[", '[',     "[", '['},
		{KVkStandardWidth,  0,     "]", ']',     "]", ']'},
		{KVkStandardWidth + (KVkStandardWidth/2),  0,     "Up", KEY_UP,     "Up", KEY_UP},
		{KVkStandardWidth + (KVkStandardWidth/2),  0,     "Dn", KEY_DOWN,     "Dn", KEY_DOWN},
		{KVkStandardWidth + (KVkStandardWidth/2),  0,     "Lf", KEY_LEFT,     "Lf", KEY_LEFT},
		{KVkStandardWidth + (KVkStandardWidth/2),  0,     "Rg", KEY_RIGHT,     "Rg", KEY_RIGHT},
		{KVkStandardWidth + (KVkStandardWidth/2),  0,     "Cpy", KEY_COPY,     "Cpy", KEY_COPY},
		{0, 0,     NULL, 0,     NULL, 0},
		{0, 0,     NULL, 0,     NULL, 0},
		{0, 0,     NULL, 0,     NULL, 0},
		{0, 0,     NULL, 0,     NULL, 0}
	},
	{
		{KVkStandardWidth,  0,     "1", KEY_P1,     "1", KEY_P1},
		{KVkStandardWidth,  0,     "2", KEY_P2,     "2", KEY_P2},
		{KVkStandardWidth,  0,     "3", KEY_P3,     "3", KEY_P3},
		{KVkStandardWidth,  0,     "4", KEY_P4,     "4", KEY_P4},
		{KVkStandardWidth,  0,     "5", KEY_P5,     "5", KEY_P5},
		{KVkStandardWidth,  0,     "6", KEY_P6,     "6", KEY_P6},
		{KVkStandardWidth,  0,     "7", KEY_P7,     "7", KEY_P7},
		{KVkStandardWidth,  0,     "8", KEY_P8,     "8", KEY_P8},
		{KVkStandardWidth,  0,     "9", KEY_P9,     "9", KEY_P9},
		{KVkStandardWidth,  0,     "0", KEY_P0,     "0", KEY_P0},
		{KVkStandardWidth,  0,     ".", KEY_PDOT,     ".", KEY_PDOT},
	},
	{
		{KVkStandardWidth + (KVkStandardWidth/2),  0,     "JL", KEY_JOYL,     "JL", KEY_JOYL},
		{KVkStandardWidth + (KVkStandardWidth/2),  0,     "JR", KEY_JOYR,     "JR", KEY_JOYR},
		{KVkStandardWidth + (KVkStandardWidth/2),  0,     "JU", KEY_JOYU,     "JU", KEY_JOYU},
		{KVkStandardWidth + (KVkStandardWidth/2),  0,     "JD", KEY_JOYD,     "JD", KEY_JOYD},
		{KVkStandardWidth + (KVkStandardWidth/2),  0,     "J1", KEY_JOYF1,     "J1", KEY_JOYF1},
		{KVkStandardWidth + (KVkStandardWidth/2),  0,     "J2", KEY_JOYF2,     "J2", KEY_JOYF2},
		{0, 0,     NULL, 0,     NULL, 0},
		{0, 0,     NULL, 0,     NULL, 0},
		{0, 0,     NULL, 0,     NULL, 0},
		{0, 0,     NULL, 0,     NULL, 0},
		{0, 0,     NULL, 0,     NULL, 0}
	}
};


CVirtualKeyboard::CVirtualKeyboard(CGameWindow& aGameWindow, CGameController& aGameController) : 
	iRect(TSize(200, 150)), iGameWindow(aGameWindow), iGameController(aGameController),
	iActiveRect(TSize(-1, -1))
{
	iBgColor =KRgbWhite;
	iFgColor =KRgbRed;
	iSelColor=KRgbBlue;

	iCurrentX = iCurrentY = 0;
}

CVirtualKeyboard::~CVirtualKeyboard()
{
	if (iFont)
		iBitmap->FbsBitmapDevice()->ReleaseFont(iFont);
	if (iFontCmd)
		iBitmap->FbsBitmapDevice()->ReleaseFont(iFontCmd);
	if (iBitmap)
		delete iBitmap;
}


CVirtualKeyboard* CVirtualKeyboard::NewL(CGameWindow& aGameWindow, CGameController& aGameController)
{
	CVirtualKeyboard*	self = new(ELeave)CVirtualKeyboard(aGameWindow, aGameController);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}



void CVirtualKeyboard::ConstructL()
{
	iBitmap = CFbsGameScreenDevice::NewL(iRect.Size(), iGameWindow.GameScreenDevice()->DisplayMode());

	TFontSpec fontSpec(KFontName, 20);
	User::LeaveIfError(iBitmap->FbsBitmapDevice()->GetNearestFontInPixels(iFont, fontSpec));

	TFontSpec fontSpec2(KFontName, 15);
	User::LeaveIfError(iBitmap->FbsBitmapDevice()->GetNearestFontInPixels(iFontCmd, fontSpec2));

	CFbsBitGc*	gc = iBitmap->FbsBitGc();
	gc->UseFont(iFont);

	// sothat we can detect our hotkey
	iGameController.AddGameControlInputHandlerL(*this);
	iGameWindow.AddScreenOrientationObserverL(*this);
	iGameController.AddGameControlRawInputHandlerL(*this);
}

void CVirtualKeyboard::SetPosition(const TPoint& aPosition)
{
	TSize	curSize = iRect.Size();
	TSize	screenSize = iGameWindow.GameScreenDevice()->Size();
	TPoint newPos = aPosition;
	if (newPos.iX < 0)
		newPos.iX = 0;
	if (newPos.iX + curSize.iWidth > screenSize.iWidth)
		newPos.iX = screenSize.iWidth - curSize.iWidth;
	if (newPos.iY < 0)
		newPos.iY = 0;
	if (newPos.iY + curSize.iHeight> screenSize.iHeight)
		newPos.iY = screenSize.iHeight - curSize.iHeight;

	iRect = TRect(newPos, curSize);
}


void CVirtualKeyboard::ScreenOrientationChanged(TOrientation aOrientation)
{
	iOrientation = aOrientation;

	DrawKeyboard();
}


TBool CVirtualKeyboard::CheckForVirtualKeyboardHotKey(TInt aKeyCode)
{
	if (IsHotKeyForVirtualKeyboard(aKeyCode))
	{
		if (!iIsActive)
			ActivateL();
		else
			Deactivate();
		
		return ETrue;
	}
	return EFalse;
}


void CVirtualKeyboard::ActivateL()
{
	iIsActive = ETrue;
	iAlphaInc = 0;
	iAlpha = 175;
	iKey = NULL;
	nbCharInCmd = 0;
	iEnter = EFalse;

	iGameWindow.AddGameScreenDrawObserverL(*this);

	DrawKeyboard();
}


void CVirtualKeyboard::Deactivate()
{
	iIsActive = EFalse;
	if (nbCharInCmd)
		iEnter = ETrue;

	// no longer need to steal pointer events
	iGameWindow.RemoveGameScreenDrawObserver(*this);
	iKey = NULL;
	ClearKeyboard();
}


void CVirtualKeyboard::PostDrawFrameToScreen(CGameScreenDevice* aDstGameScreenDevice)
{
#if 0
	if (iAlphaInc)
	{
		iAlpha += iAlphaInc;
		if (iAlpha > 255)
			iAlpha = 255;
		if (iAlpha < 0)
			iAlpha = 0;
	}

	aDstGameScreenDevice->AlphaBlend(iRect.iTl, *iBitmap, TRect(iBitmap->Size()), (TUint8) iAlpha);
#else
	if (iOrientation==EOrientationRotLeft)
		aDstGameScreenDevice->DirectDraw_16bpp_16bppRotLeft(TPoint(160, 5), *iBitmap, TPoint(0, 0), TSize(200, 150));
	else
		aDstGameScreenDevice->DirectDraw_16bpp_16bppRotRight(TPoint(160, 5), *iBitmap, TPoint(0, 0), TSize(200, 150));
#endif
}

TBool CVirtualKeyboard::WantDrawToScreen()
{
	return iIsActive;
}

void CVirtualKeyboard::HandleVirtualButtonDown(const TVirtualButton* aVirtualButton)
{
	iGameController.InjectkeyEvent(EEventKeyDown, ScanCode(aVirtualButton));
	iGameController.InjectkeyEvent(EEventKey, KeyCode(aVirtualButton));
	iButton = aVirtualButton;
}

void CVirtualKeyboard::HandleVirtualButtonUp()
{
	if (iButton)
	{
		iGameController.InjectkeyEvent(EEventKeyUp, ScanCode(iButton));
		iButton = NULL;
	}
}


TInt CVirtualKeyboard::KeyCode(const TVirtualButton* aVirtualButton)
{
	return aVirtualButton->iKeyCode;
}

TInt CVirtualKeyboard::ScanCode(const TVirtualButton* aVirtualButton)
{
	return iKeyCodesAreScanCodes ? aVirtualButton->iKeyCode : aVirtualButton->iScanCode;
}

const TVirtualButton* CVirtualKeyboard::FindVirtualButton(const TPoint& aPoint)
{
	const TVirtualButton*	button = &iVirtualButtons[0];
	while((button->iScanCode != -1) && (button->iKeyCode != -1))
	{
		TRect	rect(button->iTlX, button->iTlY, button->iBrX, button->iBrY);
		if (rect.Contains(aPoint))
			return button;
		button++;
	}
	return NULL;
}


TBool CVirtualKeyboard::CheckVirtualButton(const TPointerEvent& aPointerEvent)
{
	TBool	consumed = EFalse;

	if (aPointerEvent.iType == TPointerEvent::EButton1Down)
	{
		const TVirtualButton* virtualButton = FindVirtualButton(aPointerEvent.iPosition);
		if (virtualButton)
		{
			HandleVirtualButtonDown(virtualButton);
			consumed = ETrue;
		}
	}
	else if (aPointerEvent.iType == TPointerEvent::EButton1Up)
	{
		if (iButton)
		{
			HandleVirtualButtonUp();
			consumed = ETrue;
		}
	}
	return consumed;
}


TBool CVirtualKeyboard::ConsumeRawPointerEvent(const TPointerEvent& aPointerEvent)
{
//	RDebug::Print(_L("CVirtualKeyboard::ConsumeRawPointerEvent, type=%d, (%d, %d)"), aPointerEvent.iType, aPointerEvent.iPosition.iX, aPointerEvent.iPosition.iY);

	TPoint	p = aPointerEvent.iPosition;
	TBool	consumed = EFalse;
	if (iIsActive && iRect.Contains(p))
	{
		consumed = ETrue;
		if (aPointerEvent.iType == TPointerEvent::EButton1Down)
		{
			const TVirtualKey* virtualKey = FindVirtualKey(p - iRect.iTl);
			if (virtualKey)
				HandleVirtualKeyDown(virtualKey);
		}
		else if (aPointerEvent.iType == TPointerEvent::EButton1Up)
		{
			HandleVirtualKeyUp();
		}
	}
	else
	{
		if (iVirtualButtons)
			consumed = CheckVirtualButton(aPointerEvent);
	}
//	RDebug::Print(_L("CVirtualKeyboard::ConsumeRawPointerEvent, consume=%d)"), consumed);

	return consumed;
}


TInt CVirtualKeyboard::KeyCode(const TVirtualKey* aVirtualKey)
{
	TBool shift = iCapsMode ^ iShiftMode;
	return shift? aVirtualKey->iShiftKeyCode: aVirtualKey->iKeyCode;
}

TInt CVirtualKeyboard::ScanCode(const TVirtualKey* aVirtualKey)
{
	return iKeyCodesAreScanCodes ? aVirtualKey->iKeyCode : aVirtualKey->iScanCode;
}

void CVirtualKeyboard::HandleVirtualKeyDown(const TVirtualKey* aVirtualKey)
{
	TBool generateKeyEvent = !((aVirtualKey->iKeyCode == EKeyLeftShift) 
								|| (aVirtualKey->iKeyCode == EKeyCapsLock) || 
								(aVirtualKey->iKeyCode == KKeyCodeSym));
	if (generateKeyEvent)
	{
		iGameController.InjectkeyEvent(EEventKeyDown, ScanCode(aVirtualKey));
		iGameController.InjectkeyEvent(EEventKey, KeyCode(aVirtualKey));
	}
	iKey = aVirtualKey;
	DrawKeyboard();

	if (aVirtualKey->iKeyCode == KKeyCodeAlphaInc)
		iAlphaInc = 1;
	else if (aVirtualKey->iKeyCode == KKeyCodeAlphaDec)
		iAlphaInc = -1;
}

void CVirtualKeyboard::HandleVirtualKeyUp()
{
	if (iKey)
	{
		if (iKey->iKeyCode == EKeyLeftShift)
			iShiftMode = !iShiftMode;
		else if (iKey->iKeyCode == EKeyCapsLock)
			iCapsMode = !iCapsMode;
		else
		{
			iGameController.InjectkeyEvent(EEventKeyUp, ScanCode(iKey));
			iKey = NULL;
			if (iShiftMode)
				iShiftMode = EFalse;
		}
		DrawKeyboard();
	}
	iAlphaInc = 0;
}

void CVirtualKeyboard::HandleKeyEvent(TInt aType, const TKeyEvent& aKeyEvent)
{
	if (aType == EEventKey)
	{ TBool updateDisplay=EFalse;

		if (CheckForVirtualKeyboardHotKey(aKeyEvent.iCode))
			return;

		if (!iIsActive)
			return;

		if (aKeyEvent.iScanCode == EStdKeyDevice1)
		{
			Deactivate();
			return;
		}

		if (aKeyEvent.iScanCode == EStdKeyDevice3)
		{
			iKey = &iDefaultVirtualKeyboardTable[iCurrentY][iCurrentX];

			if (iKey->iKeyCode == KEY_CAPSLOCK)	// Caps Lock
				iCapsMode = !iCapsMode;
			if (iKey->iKeyCode == KEY_SHIFT)
				iShiftMode = !iShiftMode;
			/*else if (iKey->iKeyCode == DEVICE_SELECT_KEYCODE)
			{	//iEnter = ETrue;
				//Deactivate();
				return;
			}*/
			if (iKey->iKeyCode == KEY_BACKS)
			{	if (nbCharInCmd>0)
				{	
					szCommand[--nbCharInCmd] = 0;
					szReadableCmd[nbCharInCmd] = 0;
				}
			}
			else
			{
				szReadableCmd[nbCharInCmd]=(char)KeyCode(iKey);
				szCommand[nbCharInCmd++]=(char)iKey->iKeyCode;	// display KeyCode(iKey) but return iKey->iKeyCode to the CPC
				szCommand[nbCharInCmd] = 0;
				szReadableCmd[nbCharInCmd] = 0;
			}
			updateDisplay = ETrue;
		}
		
		TInt oldCurrentX = iCurrentX;
		TInt oldCurrentY = iCurrentY;
		if (iOrientation == EOrientationRotLeft )
		{
			if (aKeyEvent.iScanCode == EStdKeyUpArrow && iCurrentX >0)
				iCurrentX--;
			if (aKeyEvent.iScanCode == EStdKeyDownArrow && iCurrentX <KVkCols)
				iCurrentX++;
			if (aKeyEvent.iScanCode == EStdKeyLeftArrow && iCurrentY <KVkRows)
				iCurrentY++;
			if (aKeyEvent.iScanCode == EStdKeyRightArrow && iCurrentY >0)
				iCurrentY--;
		}
		else
		{
			if (aKeyEvent.iScanCode == EStdKeyDownArrow && iCurrentX >0)
				iCurrentX--;
			if (aKeyEvent.iScanCode == EStdKeyUpArrow && iCurrentX <KVkCols)
				iCurrentX++;
			if (aKeyEvent.iScanCode == EStdKeyRightArrow && iCurrentY <KVkRows)
				iCurrentY++;
			if (aKeyEvent.iScanCode == EStdKeyLeftArrow && iCurrentY >0)
				iCurrentY--;
		}

		const TVirtualKey& vk = iDefaultVirtualKeyboardTable[iCurrentY][iCurrentX];
		if (!IS_VALID_VIRTUAL_KEY(&vk))
		{
			iCurrentX = oldCurrentX;
			iCurrentY = oldCurrentY;
		}
		if (updateDisplay || iCurrentX != oldCurrentX || iCurrentY != oldCurrentY)
			DrawKeyboard();
	}
}

TBool CVirtualKeyboard::GetCommandString(char *szCmd)
{
	if (iEnter)
	{
		strcpy(szCmd,szCommand);
		iEnter = EFalse;
		return ETrue;
	}
	else
		return EFalse;
}

void CVirtualKeyboard::HandlePointerEvent(const TPointerEvent& aPointerEvent)
{
	if ((aPointerEvent.iType == TPointerEvent::EButton1Down) && iActiveRect.Contains(aPointerEvent.iPosition))
	{
		if (!iIsActive)
			ActivateL();
		else
			Deactivate();
	}
}

const TVirtualKey* CVirtualKeyboard::FindVirtualKey(const TPoint& aPoint)
{
	TInt	row = aPoint.iY / (KVkStandardHeight + KVkYMargin);
	TPoint	p(KVkXMargin, row * (KVkStandardHeight + KVkYMargin));
	if (row>=0 && row<KVkRows)
	{
		for (TInt col = 0; col<KVkCols ; col++)
		{
			const TVirtualKey& vk = iDefaultVirtualKeyboardTable[row][col];
			if (IS_VALID_VIRTUAL_KEY(&vk))
			{
				TRect	rect(p, p + TPoint (vk.iWidth, KVkStandardHeight));
				if (rect.Contains(aPoint))
					return &vk;
			}
			p.iX += KVkXMargin + vk.iWidth;
		}
	}
	return NULL;
}

void CVirtualKeyboard::ClearKeyboard()
{
	CFbsBitGc*	gc = iBitmap->FbsBitGc();
	gc->SetBrushColor(KRgbBlack);
	gc->SetBrushStyle(CGraphicsContext::ENullBrush);
	gc->Clear();
}


CFbsBitGc::TGraphicsOrientation ToGraphicsOrientation(TOrientation aOrientation)
{
	static const CFbsBitGc::TGraphicsOrientation	orientatationList[] = 
	{
		CFbsBitGc::EGraphicsOrientationNormal,
		CFbsBitGc::EGraphicsOrientationRotated90,
		CFbsBitGc::EGraphicsOrientationRotated270,
		CFbsBitGc::EGraphicsOrientationRotated180
	};
	return orientatationList[aOrientation];
}

void CVirtualKeyboard::DrawKeyboard()
{
	CFbsBitGc*	gc = iBitmap->FbsBitGc();
	gc->SetOrientation(ToGraphicsOrientation(iOrientation));
	gc->SetBrushColor(iBgColor);
	gc->SetBrushStyle(CGraphicsContext::ENullBrush);
	gc->Clear();
	gc->SetPenColor(iFgColor);
	gc->SetPenSize(TSize(1,1));
	gc->UseFont(iFont);

	TBool shift = iCapsMode ^ iShiftMode;
	TPoint	p(KVkYMargin, KVkXMargin);
	for (TInt row = 0; row<KVkRows ; row++)
	{
		p.iX = KVkXMargin;
		for (TInt col = 0; col<KVkCols ; col++)
		{
			if (col==iCurrentX && row==iCurrentY)
				gc->SetPenColor(iSelColor);
			else
				gc->SetPenColor(iFgColor);

			const TVirtualKey& vk = iDefaultVirtualKeyboardTable[row][col];
			if (IS_VALID_VIRTUAL_KEY(&vk))
			{
				TRect	rect(p, p + TPoint (vk.iWidth, KVkStandardHeight));
				if (vk.iText)
				{
					TBool isReversed = ((vk.iKeyCode == EKeyLeftShift) && (iShiftMode))
							|| ((vk.iKeyCode == EKeyCapsLock) && (iCapsMode));
					if (iKey == &vk)
						isReversed = ETrue;

					if (isReversed)
					{
						gc->SetBrushColor(iFgColor);
						gc->SetPenColor(iBgColor);
						gc->Clear(rect);
					}
					
					// draw border
					gc->DrawRect(rect);

					// draw text
					TPtrC8		text8((TUint8*) (shift ? vk.iShiftText : vk.iText));
					TBuf<32>	text;
					text.Copy(text8);
					TInt baselineOffset=(rect.Height()+iFont->AscentInPixels())/2; 
					gc->DrawText(text, rect, baselineOffset, CGraphicsContext::ECenter);

					if (isReversed)
					{
						gc->SetBrushColor(iBgColor);
						gc->SetPenColor(iFgColor);
					}
				}
			}
			p.iX += KVkXMargin + vk.iWidth;
		}
		p.iY += KVkYMargin + KVkStandardHeight;
	}

	if (nbCharInCmd)
	{
		gc->UseFont(iFontCmd);
		gc->SetPenColor(KRgbBlack);
		// draw current command
		TPtrC8		text8((TUint8*) szReadableCmd);
		TBuf<32>	text;
		text.Copy(text8);
		//TInt baselineOffset=(rect.Height()+iFont->AscentInPixels())/2; 
		gc->DrawText(text, TPoint(5,146));
	}
	//gc->SetOrientation(ToGraphicsOrientation(iOrientation));
}

/****************************************************
 *
 * IMPLEMENTATION OF TCPCControlInputHandler
 *
 ****************************************************/


//const TInt KMaxLastDeltas = 3;
const TInt KDeferJogdialUpEventsDeltaFrames = 3;
#define MAX_CMD_STRING_SIZE	256

/********************************************************* !NAME! **************
* Nom : stToucheClav
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Structures
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Structure clavier CPC
*
********************************************************** !0! ****************/
typedef struct
    {
    UBYTE Port;
    UBYTE Ligne;
    } stToucheClav;

class TCPCControlInputHandler : public MGameControlInputHandler, public MScreenOrientationObserver, public MGameScreenDrawObserver
{
public:
	TCPCControlInputHandler();
#if 0
	void PostKeyEvent(TInt aKeyCode, TBool aUpEvent);
	TInt MapToKeyCode(TInt aScancode);
#else
	void PostKeyEvent(stToucheClav aKeyCode, TBool aUpEvent);
	stToucheClav MapToKeyCode(TInt aScancode);
#endif

public:			// implements MScreenOrientationObserver
	virtual void ScreenOrientationChanged(TOrientation aOrientation);

public:		// implements MGameControlInputHandler
	virtual void HandleKeyEvent(TInt aType, const TKeyEvent& aKeyEvent);
	TBool CheckDirectCmd(TInt aType, const TKeyEvent& aKeyEvent);
	virtual void HandlePointerEvent(const TPointerEvent& /*aPointerEvent*/){};

public:		// implements MGameScreenDrawObserver
	virtual TBool	WantDrawToScreen() {return EFalse;}
	virtual void	PreDrawFrameToScreen(CGameScreenDevice* aDstGameScreenDevice);
	virtual void	PostDrawFrameToScreen(CGameScreenDevice* /*aDstGameScreenDevice*/){};

	// Command strings
	void SetCommandString(char *szCmdStr);

protected:
	static const TInt sKeyMapTableRotLeft[][2];
	static const TInt sKeyMapTableRotRight[][2];
	
	TBool		iIsRotLeft;
	TInt		iJogdialDownInFrame[2];
	TInt		iFrames;

	TBool		isMovingForward;

	char		szCmdString[MAX_CMD_STRING_SIZE];
	TInt		iPosInString;
	TBool		CmdKeyUp;
	TBool		ShiftKeyUp;
};
TCPCControlInputHandler			sCPCControlInputHandler;

/********************************************************* !NAME! **************
* Nom : TrKey
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Transformations touches Phone -> CPC
*
********************************************************** !0! ****************/
const stToucheClav TrKey[ 8 ] =	// test dans un 1er temps
{
    { 0xFB, 0x09 },           // JoyLeft
    { 0xF7, 0x09 },           // JoyRight
    { 0xFD, 0x09 },           // JoyDown
    { 0xFE, 0x09 },           // JoyUp
    { 0xEF, 0x09 },           // JoyFire1
    { 0xDF, 0x09 },           // JoyFire2
    { 0x7F, 0x09 },           // DEL
    { 0xFD, 0x08 },           // '2'
};

typedef struct
{
    UBYTE Char;
    UBYTE Port;
    UBYTE Ligne;
} stCharMapping;

#define NUM_MAPPED_CHARS	76
const stCharMapping trChar[ NUM_MAPPED_CHARS ] =
{
    { 'Q', 0xF7, 0x08 },           // 'A'
    { 'B', 0xBF, 0x06 },           // 'B'
    { 'C', 0xBF, 0x07 },           // 'C'
    { 'D', 0xDF, 0x07 },           // 'D'
    { 'E', 0xFB, 0x07 },           // 'E'
    { 'F', 0xDF, 0x06 },           // 'F'
    { 'G', 0xEF, 0x06 },           // 'G'
    { 'H', 0xEF, 0x05 },           // 'H'
    { 'I', 0xF7, 0x04 },           // 'I'
    { 'J', 0xDF, 0x05 },           // 'J'
    { 'K', 0xDF, 0x04 },           // 'K'
    { 'L', 0xEF, 0x04 },           // 'L'
    //{ 'M', 0xDF, 0x03 },           // 'M'
	{ 'M', 0xBF, 0x04 },           // 'M'
    { 'N', 0xBF, 0x05 },           // 'N'
    { 'O', 0xFB, 0x04 },           // 'O'
    { 'P', 0xF7, 0x03 },           // 'P'
    { 'A', 0xDF, 0x08 },           // 'Q'
    { 'R', 0xFB, 0x06 },           // 'R'
    { 'S', 0xEF, 0x07 },           // 'S'
    { 'T', 0xF7, 0x06 },           // 'T'
    { 'U', 0xFB, 0x05 },           // 'U'
    { 'V', 0x7F, 0x06 },           // 'V'
    { 'Z', 0x7F, 0x08 },           // 'W'
    { 'X', 0x7F, 0x07 },           // 'X'
    { 'Y', 0xF7, 0x05 },           // 'Y'
    { 'W', 0xF7, 0x07 },           // 'Z'
    /*{ '0', 0x7F, 0x01 },           // P'0'
    { '1', 0xDF, 0x01 },           // P'1'
    { '2', 0xBF, 0x01 },           // P'2'
    { '3', 0xDF, 0x00 },           // P'3'
    { '4', 0xEF, 0x02 },           // P'4'
    { '5', 0xEF, 0x01 },           // P'5'
    { '6', 0xEF, 0x00 },           // P'6'
    { '7', 0xFB, 0x01 },           // P'7'
    { '8', 0xF7, 0x01 },           // P'8'
    { '9', 0xF7, 0x00 },           // P'9'*/
    { '0', 0xFE, 0x04 },           // '0'
    { '1', 0xFE, 0x08 },           // '1'
    { '2', 0xFD, 0x08 },           // '2'
    { '3', 0xFD, 0x07 },           // '3'
    { '4', 0xFE, 0x07 },           // '4'
    { '5', 0xFD, 0x06 },           // '5'
    { '6', 0xFE, 0x06 },           // '6'
    { '7', 0xFD, 0x05 },           // '7'
    { '8', 0xFE, 0x05 },           // '8'
    { '9', 0xFD, 0x04 },           // '9'
	{ KEY_ENTER, 0xFB, 0x02 },     // ENTER
	{ KEY_CAPSLOCK, 0xBF, 0x08 },  // CAPS LOCK
	{ KEY_SHIFT, 0xDF, 0x02 },	   // SHIFT
    { ' ', 0x7F, 0x05 },           // ' '
	{ '@', 0xFB, 0x03 },			// '@'
	{ '-', 0xFD, 0x03 },			// '-'
	{ ':', 0xDF, 0x03 },			// ':'
	{ ';', 0xEF, 0x03 },			// ';'
	{ ',', 0x7F, 0x04 },			// ','
	{ '.', 0x7F, 0x03 },			// '.'
	{ '/', 0xBF, 0x03 },			// '/'
	{ '\\', 0xBF, 0x02 },			// '\'
	{ '[', 0xFD, 0x02 },			// '['
	{ ']', 0xF7, 0x02 },			// ']'
	{ '^', 0xFE, 0x03 },			// '^'
	{ KEY_ESC, 0xFB, 0x08 },		// ESC
	{ KEY_CTRL, 0x7F, 0x02 },		// CTRL
	{ KEY_TAB, 0xEF, 0x08 },		// TAB
	{ KEY_JOYL, 0xFB, 0x09 },		// JOY left
	{ KEY_JOYR, 0xF7, 0x09 },		// JOY right
	{ KEY_JOYU, 0xFE, 0x09 },		// JOY up
	{ KEY_JOYD, 0xFD, 0x09 },		// JOY down
	{ KEY_JOYF1, 0xEF, 0x09 },		// JOY fire1
	{ KEY_JOYF2, 0xDF, 0x09 },		// JOY fire2
	{ KEY_LEFT, 0xFE, 0x01 },		// left
	{ KEY_RIGHT, 0xFD, 0x00 },		// right
	{ KEY_UP, 0xFE, 0x00 },			// up
	{ KEY_DOWN, 0xFB, 0x00 },		// down
	{ KEY_COPY, 0xFD, 0x01 },		// COPY
    { KEY_P0, 0x7F, 0x01 },         // P'0'
    { KEY_P1, 0xDF, 0x01 },         // P'1'
    { KEY_P2, 0xBF, 0x01 },         // P'2'
    { KEY_P3, 0xDF, 0x00 },         // P'3'
    { KEY_P4, 0xEF, 0x02 },         // P'4'
    { KEY_P5, 0xEF, 0x01 },         // P'5'
    { KEY_P6, 0xEF, 0x00 },         // P'6'
    { KEY_P7, 0xFB, 0x01 },         // P'7'
    { KEY_P8, 0xF7, 0x01 },         // P'8'
    { KEY_P9, 0xF7, 0x00 },         // P'9'
	{ KEY_PDOT, 0x7F, 0x00 },        // P'.'
};

/********************************************************* !NAME! **************
* Nom : JoyKey
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Transformations touches Phone -> Joystick CPC
*
********************************************************** !0! ****************/
/*const stToucheClav JoyKey[ 256 ];

extern "C"
{
	int JoyLeft;
	int JoyRight;
	int JoyUp;
	int JoyDown;
	int JoyFire1;
	int JoyFire2;

	stToucheClav JoyKey[ 256 ];
}*/
stCharMapping sKeyMapTable[MAX_PHONE_KEYS] =
{
	{EStdKeyUpArrow, 0xFB, 0x09 },		// JoyLeft
	{EStdKeyDownArrow, 0xF7, 0x09 },	// JoyRight
	{EStdKeyLeftArrow, 0xFD, 0x09 },	// JoyDown
	{EStdKeyRightArrow, 0xFE, 0x09 },	// JoyUp
	//{0x2a, 0xEF, 0x09 },				// JoyFire1
	{EStdKeyDevice3, 0xEF, 0x09 },		// JoyFire1
	{EStdKeyHash, 0xDF, 0x09},			// JoyFire2
	{EStdKeyBackspace, 0x7F, 0x09},		// DEL
	{0, 0}
};

#define JOYLEFT		0
#define JOYRIGHT	1
#define JOYDOWN		2
#define JOYUP		3
#define JOYFIRE1	4
#define JOYFIRE2	5
const TInt TCPCControlInputHandler::sKeyMapTableRotLeft[][2] = 
{
	/*{EStdKeyBackspace, KEY_ESCAPE},
	{EStdKeyDevice3, KEY_ENTER},	
	*/
	{EStdKeyUpArrow, JOYLEFT },
	{EStdKeyDownArrow, JOYRIGHT },
	{EStdKeyLeftArrow, JOYDOWN },
	{EStdKeyRightArrow, JOYUP },
	{/*EStdKeyDevice3*/0x2a, JOYFIRE1 },
	{EStdKeyHash, JOYFIRE2},
	{EStdKeyBackspace,6},
/*
	{EStdKeyApplication0, 'F'},
	{EStdKeyApplication1, 'G'},
	{EStdKeyDevice0, 'y'},

	{EStdKeyTab, KEY_TAB},
*/
	{0, 0}
};

const TInt TCPCControlInputHandler::sKeyMapTableRotRight[][2] = 
{/*
	{GAME_LEFT_SCANCODE, KEY_LEFTARROW},
	{GAME_RIGHT_SCANCODE, KEY_RIGHTARROW},
	{GAME_UP_SCANCODE, KEY_UPARROW},
	{GAME_DOWN_SCANCODE, KEY_DOWNARROW},

	{EStdKeyEscape, KEY_ESCAPE},
	{EStdKeyEnter, KEY_ENTER},

	{DEVICE_DOWN_SCANCODE, 'A'},
	{DEVICE_UP_SCANCODE, 'B'},
	{DEVICE_RIGHT_SCANCODE, 'C'},
	{DEVICE_LEFT_SCANCODE, 'D'},
	{DEVICE_SELECT_KEYCODE, 'E'},

	{EStdKeyApplication0, 'F'},
	{EStdKeyApplication1, 'G'},

	{EStdKeyTab, KEY_TAB},
*/
	{0, 0}
};

typedef struct
{
	TInt iStdScanCode;
	char szCmdString[255];
} stDirectCommand;

stDirectCommand trDirectCommands[MAX_PHONE_KEYS] =
{
	{ 0x32, "RUN\"" },
	//{ 0x33, "RUN\"FRUITY" },

	{ 0, "" }
};

TCPCControlInputHandler::TCPCControlInputHandler()
{
	iJogdialDownInFrame[0] = -1 - KDeferJogdialUpEventsDeltaFrames;
	iJogdialDownInFrame[1] = -1 - KDeferJogdialUpEventsDeltaFrames;

	iPosInString = -1;
	CmdKeyUp = false;
	ShiftKeyUp = false;
}

void TCPCControlInputHandler::ScreenOrientationChanged(TOrientation aOrientation)
{
	iIsRotLeft = (aOrientation == EOrientationRotLeft);
}

#if 0
TInt TCPCControlInputHandler::MapToKeyCode(TInt aScancode)
#else
stToucheClav TCPCControlInputHandler::MapToKeyCode(TInt aScancode)
#endif
{
	int i;
#if 0
	if (iIsRotLeft)
	{
		for (i = 0; sKeyMapTableRotLeft[i][0]; i++)
			if (sKeyMapTableRotLeft[i][0] == aScancode)
				return sKeyMapTableRotLeft[i][1];
	}
	else
	{
		for (i = 0; sKeyMapTableRotRight[i][0]; i++)
			if (sKeyMapTableRotRight[i][0] == aScancode)
				return sKeyMapTableRotRight[i][1];
	}
	//return aScancode;
	return -1;

#else

	stToucheClav retCode;
	for (i = 0; sKeyMapTable[i].Char; i++)
		if (sKeyMapTable[i].Char == aScancode)
		{	
			retCode.Ligne = sKeyMapTable[i].Ligne;
			retCode.Port = sKeyMapTable[i].Port;
			return retCode;
		}
	retCode.Ligne = 0xFF;
	return retCode;
#endif
}

TBool TCPCControlInputHandler::CheckDirectCmd(TInt aType, const TKeyEvent& aKeyEvent)
{
	int i;

	for (i = 0; trDirectCommands[i].iStdScanCode; i++)
		if (trDirectCommands[i].iStdScanCode == aKeyEvent.iScanCode)
		{
			if (aType == EEventKeyDown)
				SetCommandString(trDirectCommands[i].szCmdString);
			return true;
		}

	return false;
}

void TCPCControlInputHandler::HandleKeyEvent(TInt aType, const TKeyEvent& aKeyEvent)
{
	if (iPosInString>=0)	// a command string is "typed"
		return;

	if (CheckDirectCmd(aType, aKeyEvent))
		return;

	if (aType == EEventKey && aKeyEvent.iScanCode == EStdKeyDevice1)
		//finMain = 1;
		Quit();

	if (aType == EEventKeyDown)
	{	// MR
		/*if (aKeyEvent.iScanCode == DEVICE_UP_SCANCODE)
		{
			iJogdialDownInFrame[0] = iFrames;
		}
		else if (aKeyEvent.iScanCode == DEVICE_DOWN_SCANCODE)
		{
			iJogdialDownInFrame[1] = iFrames;
		}*/
		PostKeyEvent(MapToKeyCode(aKeyEvent.iScanCode), EFalse);
	}
	else if (aType == EEventKeyUp)
	{
		// MR
		//if ((aKeyEvent.iScanCode != DEVICE_UP_SCANCODE) && (aKeyEvent.iScanCode != DEVICE_DOWN_SCANCODE))
			PostKeyEvent(MapToKeyCode(aKeyEvent.iScanCode), ETrue);
	}
/*	else if (aType == EEventKey)
	{
		if (isMovingForward && ( (aKeyEvent.iCode==1) || (aKeyEvent.iCode==2) ))
		{
			event_t ev;
			ev.type = ev_joystick;
			ev.data1 = 0;
			ev.data3 = 0;
			ev.data2 = (aKeyEvent.iCode==1) ? 75 : -75;
	//		RDebug::Print(_L("ev_mouse: aX=%d, aY=%d, mouseX=%d, mouseY=%d"), aX, aY, ev.data2, ev.data3);
			D_PostEvent(&ev);
		}
		if (aKeyEvent.iCode == 'p')
		{
			ToggleProfiler();
		}
	}*/
}

#if 0
void TCPCControlInputHandler::PostKeyEvent(TInt aKeyCode, TBool aUpEvent)
#else
void TCPCControlInputHandler::PostKeyEvent(stToucheClav aKeyCode, TBool aUpEvent)
#endif
{
#if 0
	if (aKeyCode!=-1)
	{
        if (aUpEvent)
		{
			clav[ TrKey[ aKeyCode ].Ligne ] |= ~TrKey[ aKeyCode ].Port;
			//clav[ JoyKey[ aKeyCode ].Ligne ] |= ~JoyKey[ aKeyCode ].Port;
		}
		else
		{
            clav[ TrKey[ aKeyCode ].Ligne ] &= TrKey[ aKeyCode ].Port;
            //clav[ JoyKey[ aKeyCode ].Ligne ] &= JoyKey[ aKeyCode ].Port;
		}
	}
#else
	if (aKeyCode.Ligne!=0xFF)
	{
        if (aUpEvent)
		{
			clav[ aKeyCode.Ligne ] |= ~aKeyCode.Port;
		}
		else
		{
            clav[ aKeyCode.Ligne ] &= aKeyCode.Port;
		}
	}
#endif
}


void TCPCControlInputHandler::PreDrawFrameToScreen(CGameScreenDevice* /*aDstGameScreenDevice*/)
{
	iFrames++;
// MR
/*	if (iJogdialDownInFrame[0] + KDeferJogdialUpEventsDeltaFrames == iFrames)
		PostKeyEvent(MapToKeyCode(DEVICE_UP_SCANCODE), ETrue);
	if (iJogdialDownInFrame[1] + KDeferJogdialUpEventsDeltaFrames == iFrames)
		PostKeyEvent(MapToKeyCode(DEVICE_DOWN_SCANCODE), ETrue);*/

	if (iPosInString >=0)
	{
		if (szCmdString[iPosInString]==KEY_SHIFT)
			CmdKeyUp = ShiftKeyUp;

		for (int i=0;i<NUM_MAPPED_CHARS;i++)
		{
			if (trChar[i].Char == szCmdString[iPosInString])
			{	
				if (!CmdKeyUp)
				{	
					clav[ trChar[ i ].Ligne ] &= trChar[ i ].Port;
					CmdKeyUp=true;
				}
				else
				{	
					clav[ trChar[ i ].Ligne ] |= ~trChar[ i ].Port;
					CmdKeyUp=false;
				}
				break;
			}
		}

		if (szCmdString[iPosInString]==KEY_SHIFT)
		{	
			ShiftKeyUp = CmdKeyUp;
			CmdKeyUp=false;
			iPosInString++;
		}
		else if (!CmdKeyUp)
			iPosInString++;
		
		if (iPosInString>strlen(szCmdString))
			iPosInString = -1;
	}
}

void TCPCControlInputHandler::SetCommandString(char *_szCmdStr)
{
	if (strlen(_szCmdStr)<MAX_CMD_STRING_SIZE)
	{
		for (int i=0,c=0;i<strlen(_szCmdStr);i++,c++)
		{
			if (_szCmdStr[i]=='\"')
			{
				szCmdString[c++]=KEY_SHIFT;
				szCmdString[c++]='2';
				szCmdString[c]=KEY_SHIFT;
			}
			else if (_szCmdStr[i]=='')
				szCmdString[c]=KEY_ENTER;
			else
				szCmdString[c]=_szCmdStr[i];
		}
		szCmdString[c] = 0;
		iPosInString = 0;
	}
}


/***********************************************
 *
 * CGameWindow
 *
 ***********************************************/


CGameWindow::CGameWindow()
{
}


CGameWindow::~CGameWindow()
{
	delete iInfoMgr;
	iInfoMgr = NULL;

	DestructMenuL();
	DestructFileSelectionsL();
	DestructVirtualKeyboard();
	DestructGameController();
	DestructScreenDevice();
	DestructWsResources();

	delete iScreenOrientationObservers;
	iScreenOrientationObservers = NULL;

	delete iGameScreenDrawObservers;
	iGameScreenDrawObservers = NULL;
}


CGameWindow* CGameWindow::NewL(const TDesC& aGameName)
{
	CGameWindow*	self = new(ELeave)CGameWindow;
	CleanupStack::PushL(self);
	self->ConstructL(aGameName);
	CleanupStack::Pop();
	return self;
}


void CGameWindow::ConstructL(const TDesC& aGameName)
{
	iGameName = aGameName.Left(iGameName.MaxLength());

	iGameScreenDrawObservers = new(ELeave)CArrayPtrFlat<MGameScreenDrawObserver>(5);
	iScreenOrientationObservers = new(ELeave)CArrayPtrFlat<MScreenOrientationObserver>(5);

	ConstructWsResourcesL();
	ConstructScreenDeviceL();
	ConstructGameControllerL();
	ConstructVirtualKeyboardL();
	ConstructFileSelectionsL();
	ConstructMenuL();
	ClearScreen();

	SetGameScreenDrawer(&iDirectBitmapDrawer);
	iGamePaner.ActivateL(this, iGameController, EOrientationNormal);

	iInfoMgr = CInfoMgr::NewL(*this, *iGameController);

	iMemoryGameScreenDevice = 	CMemGameScreenDevice::NewL(iGameScreenDevice->Size(), iGameScreenDevice->DisplayMode());
	CaptureKeyEvents();
}

void CGameWindow::CaptureKeyEvents()
{
	// MR : /*, 1*/
	if (!iCapturedButtons[0])
		iCapturedButtons[0] = iWsWindowGroup.CaptureKey(EKeyApplication0, 0, 0/*, 1*/);
	if (!iCapturedButtons[1])
		iCapturedButtons[1] = iWsWindowGroup.CaptureKey(EKeyApplication1, 0, 0/*, 1*/);
	if (!iCapturedButtons[2])
		iCapturedButtons[2] = iWsWindowGroup.CaptureKey(EQuartzKeyBack, 0, 0/*, 1*/);
	if (!iCapturedButtons[3])
		iCapturedButtons[3] = iWsWindowGroup.CaptureKey(EQuartzKeyMenu2, 0, 0/*, 1*/);
	if (!iCapturedButtons[4])
		iCapturedButtons[4] = iWsWindowGroup.CaptureKey(EKeyDevice0, 0, 0/*, 1*/);
}

void CGameWindow::ReleaseKeyEvents()
{
	for (TInt i=0 ; i<KMaxCapturedButtons ; i++)
	{
		if (iCapturedButtons[i])
		{
			iWsWindowGroup.CancelCaptureKey(iCapturedButtons[i]);
			iCapturedButtons[i] = 0;
		}
	}
}

void CGameWindow::Hide()
{
	ReleaseKeyEvents();
	iWsWindowGroup.SetOrdinalPosition(KMinTInt, KMinTInt);
	iWsWindow.SetVisible(EFalse);

	TRect	rect(iWsWindow.Position(), iWsWindow.Size());
	iWindowGc->Activate(iWsWindow);
	iWsWindow.Invalidate();
	iWsWindow.BeginRedraw(rect);
	iWsWindow.EndRedraw();
	iWindowGc->Deactivate();
	
	iWsSession.Flush();
	iWasPaused = ETrue;
}

void CGameWindow::Show()
{
	iWsWindowGroup.SetOrdinalPosition(0, 0);
	iWsWindow.SetVisible(ETrue);
	CaptureKeyEvents();
}


void CGameWindow::ConstructWsResourcesL()
{
	TInt	error;

	error = iWsSession.Connect();
	User::LeaveIfError(error);
	iWsScreen=new(ELeave) CWsScreenDevice(iWsSession);
	User::LeaveIfError(iWsScreen->Construct());
	User::LeaveIfError(iWsScreen->CreateContext(iWindowGc));

	iWsWindowGroup=RWindowGroup(iWsSession);
	User::LeaveIfError(iWsWindowGroup.Construct((TUint32)&iWsWindowGroup));
	iWsWindowGroup.SetOrdinalPosition(0);
	iWsWindowGroup.SetName(iGameName);

	iWsWindow=RWindow(iWsSession);
	User::LeaveIfError(iWsWindow.Construct(iWsWindowGroup, (TUint32)&iWsWindow));
	iWsWindow.Activate();
	iWsWindow.SetSize(iWsScreen->SizeInPixels());

#ifdef __TEST_DIRECT_SCREEN_ACCESS__
	// you can then click on the app picker to force something to move in front of the game 
	iWsWindow.SetPosition(TPoint(0, 4));
#endif

	iWsWindow.SetVisible(ETrue);
	iWsWindow.SetPointerGrab(ETrue);
	iWsWindow.PointerFilter(EPointerFilterDrag, 0);

		/*
		 * Tell the Window Server not to mess about with our process priority
		 * Also, because of the way legacy games are written, they never sleep
		 * and thus never voluntarily yield the CPU. We set our process priority
		 * to EPriorityForeground and hope that a Telephony application on
		 * this device runs at EPriorityForeground as well. If not, tough! ;-)
		 */
	iWsSession.ComputeMode(RWsSession::EPriorityControlDisabled);
	RProcess		me;
	me.SetPriority(EPriorityForeground);
}


void CGameWindow::DestructWsResources()
{
	if (iWsWindow.WsHandle())
		iWsWindow.Close();

	if (iWsWindowGroup.WsHandle())
		iWsWindowGroup.Close();

	delete iWindowGc;
	iWindowGc = NULL;

	delete iWsScreen;
	iWsScreen = NULL;

	if (iWsSession.WsHandle())
		iWsSession.Close();
}


void CGameWindow::ConstructGameControllerL()
{
	iGameController = CGameController::NewL(iWsSession);
	AddGameScreenDrawObserverL(*iGameController);
	AddScreenOrientationObserverL(*iGameController);
}

void CGameWindow::DestructGameController()
{
	delete iGameController;
	iGameController = NULL;
}

void CGameWindow::ConstructScreenDeviceL()
{
	TDisplayMode	mode = iWsScreen->DisplayMode();
	TSize			size = iWsScreen->SizeInPixels();

	//DEB_Print(size.iWidth, size.iHeight, mode, 0);

//#ifdef __WINS__
#if 1	// MR: CHwFbGameScreenDevice a chie sur le SX1
	iUseWServ = ETrue;
#endif

	if (iUseWServ)
	{
			/*
			 * For WINS we emulate using a Fbs bitmap which we blit to the window's device
			 * context. You can test most behaviour by:
			 * 1) adjusting the size and display mode in the CFbsGameScreenDevice::NewL
			 * 2) configuring the emulator for the appropriate size and mode
			 */
		iGameScreenDevice = CFbsGameScreenDevice::NewL(size, mode);
	}
	else
	{
		iGameScreenDevice = CHwFbGameScreenDevice::NewL(size, mode);
	}

	iGameDirectScreenAccess = CGameDirectScreenAccess::NewL(*this);
	AddGameScreenDrawObserverL(*iGameDirectScreenAccess);
}


void CGameWindow::DestructScreenDevice()
{
	delete iGameScreenDevice;
	iGameScreenDevice = NULL;

	delete iGameDirectScreenAccess;
	iGameDirectScreenAccess = NULL;
}


void CGameWindow::ConstructVirtualKeyboardL()
{
	iVirtualKeyboard = CVirtualKeyboard::NewL(*this, *iGameController);
}

void CGameWindow::DestructVirtualKeyboard()
{
	delete iVirtualKeyboard;
	iVirtualKeyboard = NULL;
}

void CGameWindow::ConstructFileSelectionsL()
{
	iFileSelectionDSK = CFileSelection::NewL(*this, *iGameController, FILE_DSK);
}

void CGameWindow::DestructFileSelectionsL()
{
	delete iFileSelectionDSK;
	iFileSelectionDSK = NULL;
}

void CGameWindow::ConstructMenuL()
{
	iMenu = CMenuPage::NewL(*this, *iGameController, iFileSelectionDSK, iVirtualKeyboard);
}

void CGameWindow::DestructMenuL()
{
	delete iMenu;
	iMenu = NULL;
}

//#define SKIPFRAME	4
int SKIPFRAME=0;
void CGameWindow::GameFrameTick(CGameScreenDevice* aSrcGameScreenDevice)
{
	TBool				useMemoryScreenDevice = EFalse;
	CGameScreenDevice*	screenDevice = iGameScreenDevice;
	static int frameskip=0;
	static int inactivity=0;

		/*
		 * If any of the screen draw observers want to draw something to screen
		 * we will use the memory screen device first. This will prevent the usual
		 * screen flicker.
		 * Also, if we used to use the memory screen device and this frame we don't
		 * then we use it one more time to properly clear the screen.
		 */
	for (TInt ii=0 ; ii<iGameScreenDrawObservers->Count() && !useMemoryScreenDevice; ii++)
		useMemoryScreenDevice = (*iGameScreenDrawObservers)[ii]->WantDrawToScreen();

	if (useMemoryScreenDevice || iUsedMemoryScreenDevice || iWasPaused)
	{
		screenDevice = iMemoryGameScreenDevice;
		iMemoryGameScreenDevice->Clear();
		iWasPaused = EFalse;
	}

	for (TInt i=0 ; i<iGameScreenDrawObservers->Count() ; i++)
		(*iGameScreenDrawObservers)[i]->PreDrawFrameToScreen(screenDevice);

	if (iGameScreenDrawer /*&& !frameskip/*frameskip!=SKIPFRAME-1*/)		// !frameskip = joue 1 frame sur SKIPFRAME ... frameskip!=SKIPFRAME-1 = skip 1 frame toutes les SKIPFRAME
		iGameScreenDrawer->DrawFrame(screenDevice, aSrcGameScreenDevice);
	//frameskip++; frameskip=frameskip%SKIPFRAME;

	for (TInt j=0 ; j<iGameScreenDrawObservers->Count() ; j++)
		(*iGameScreenDrawObservers)[j]->PostDrawFrameToScreen(screenDevice);

	char szCmd[256];
	if (iVirtualKeyboard->GetCommandString(szCmd))
	{	for (TInt i=0;i<strlen(szCmd);i++)
			szCmd[i]=(char)toupper(szCmd[i]);
		sCPCControlInputHandler.SetCommandString(szCmd);
	}

	if (useMemoryScreenDevice || iUsedMemoryScreenDevice)
		iGameScreenDevice->DirectDraw(TPoint(0,0), *iMemoryGameScreenDevice, iGameScreenDevice->Size());

	iUsedMemoryScreenDevice = useMemoryScreenDevice;

	if (iUseWServ)
		WsBlitBitmap((CFbsGameScreenDevice*) iGameScreenDevice);

	if (++inactivity>170)	// backlight always on
	{
		inactivity=0;
		User::ResetInactivityTime();
	}

}

void CGameWindow::ClearScreen()
{
	iGameScreenDevice->Clear();

	if (iUseWServ)
		WsBlitBitmap((CFbsGameScreenDevice*) iGameScreenDevice, TRect(iGameScreenDevice->Size()));
}


void CGameWindow::SetGameScreenDrawer(MGameScreenDrawer* aGameScreenDrawer)
{
	if (aGameScreenDrawer)
		iGameScreenDrawer = aGameScreenDrawer;
	else
		iGameScreenDrawer = &iDirectBitmapDrawer;
}



void CGameWindow::WsBlitBitmap(CFbsGameScreenDevice* aFbsGameScreenDevice)
{
	WsBlitBitmap(aFbsGameScreenDevice, GameVisibleArea());
}

void CGameWindow::WsBlitBitmap(CFbsGameScreenDevice* aFbsGameScreenDevice, const TRect& aRect)
{
	iWindowGc->Activate(iWsWindow);

	iWsWindow.Invalidate(aRect);
	iWsWindow.BeginRedraw(aRect);

	if (aFbsGameScreenDevice)
	{
		iWindowGc->BitBlt(aRect.iTl, aFbsGameScreenDevice->FbsBitmap(), aRect);
	}

	iWsWindow.EndRedraw();
	iWindowGc->Deactivate();
	iWsSession.Flush();
}


void CGameWindow::AddScreenOrientationObserverL(MScreenOrientationObserver& aObserver)
{
	iScreenOrientationObservers->AppendL(&aObserver);
	aObserver.ScreenOrientationChanged(iOrientation);
}

void CGameWindow::SetScreenOrientation(TOrientation aOrientation)
{
	iOrientation = aOrientation;
	for (TInt i=0 ; i<iScreenOrientationObservers->Count() ; i++)
		(*iScreenOrientationObservers)[i]->ScreenOrientationChanged(iOrientation);
}



/*****************************************
 *
 * CGameController
 *
 *****************************************/

CGameController* CGameController::NewL(RWsSession& aWsSession)
{
	CGameController*	self = new(ELeave)CGameController(aWsSession);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}

CGameController::~CGameController()
{
	delete iGameControlInputHandlers;
	delete iGameControlRawInputHandlers;
}

static TGameScanCodeTable	sP800GameScanCodeTable[] = 
{
#ifndef PROJECT_S60CPC
	{DEVICE_LEFT_SCANCODE, GAME_BUTTON2_SCANCODE},
	{DEVICE_RIGHT_SCANCODE, GAME_BUTTON2_SCANCODE},
	{DEVICE_UP_SCANCODE, GAME_BUTTON1_SCANCODE},
	{DEVICE_DOWN_SCANCODE, GAME_BUTTON1_SCANCODE},
	{DEVICE_SELECT_SCANCODE, GAME_BUTTON2_SCANCODE},
#endif
	{0, 0}
};

static TGameKeyCodeTable	sP800GameKeyCodeTable[] = 
{
#ifndef PROJECT_S60CPC
	{DEVICE_LEFT_KEYCODE, GAME_BUTTON2_KEYCODE},
	{DEVICE_RIGHT_KEYCODE, GAME_BUTTON2_KEYCODE},
	{DEVICE_UP_KEYCODE, GAME_BUTTON1_KEYCODE},
	{DEVICE_DOWN_KEYCODE, GAME_BUTTON1_KEYCODE},
	{DEVICE_SELECT_KEYCODE, GAME_BUTTON2_KEYCODE},
#endif
	{0, 0}
};

void CGameController::ConstructL()
{
	iGameControlInputHandlers = new(ELeave)CArrayPtrFlat<MGameControlInputHandler>(5);
	iGameControlRawInputHandlers = new(ELeave)CArrayPtrFlat<MGameControlRawInputHandler>(5);

	SetGameKeymapping(sP800GameScanCodeTable[0], sP800GameKeyCodeTable[0]);

	StartWServEvents();
}

void CGameController::SetGameKeymapping(TGameScanCodeTable& aGameScanCodeTable, TGameKeyCodeTable& aGameKeyCodeTable)
{
	iGameScanCodeTable = &aGameScanCodeTable;
	iGameKeyCodeTable = &aGameKeyCodeTable;
}


void CGameController::StartWServEvents()
{
	iWsEventStatus = KRequestPending;
	iWsSession.EventReady(&iWsEventStatus);

	iRedrawEventStatus = KRequestPending;
	iWsSession.RedrawReady(&iRedrawEventStatus);
}

void CGameController::CancelWServEvents()
{
	if (iWsEventStatus != KRequestPending)
		iWsSession.EventReadyCancel();

	if (iRedrawEventStatus != KRequestPending)
		iWsSession.RedrawReadyCancel();
}


void CGameController::PollWServEvents()
{
	while (iWsEventStatus != KRequestPending)
	{
		iWsSession.GetEvent(iWsEvent);
		HandleWsEvent(iWsEvent);
		iWsEventStatus = KRequestPending;
		iWsSession.EventReady(&iWsEventStatus);
	}
}

void CGameController::HandleWsEvent(const TWsEvent& aWsEvent)
{
	if ((aWsEvent.Type() == EEventKeyDown) || (aWsEvent.Type() == EEventKey) || (aWsEvent.Type() == EEventKeyUp))
	{
		HandleKeyEvent(aWsEvent.Type(), *aWsEvent.Key());
	} 
	else if (aWsEvent.Type() == EEventPointer)
	{
		HandlePointerEvent(*aWsEvent.Pointer());
	}
	else
	{
//		RDebug::Print(_L("HandleWsEvent, event=%d"), aWsEvent.Type());
	}

}


void CGameController::ScreenOrientationChanged(TOrientation aOrientation)
{
	iOrientation = aOrientation;
}


void CGameController::PostDrawFrameToScreen(CGameScreenDevice* /*aDstGameScreenDevice*/)
{
	PollWServEvents();
}

		/*
		 * Raw pointer events
		 */
void CGameController::HandlePointerEvent(const TPointerEvent& aPointerEvent)
{
//	RDebug::Print(_L("CGameController::HandlePointerEvent, type=%d, (%d, %d)"), aPointerEvent.iType, aPointerEvent.iPosition.iX, aPointerEvent.iPosition.iY);
	TBool	consumed = EFalse;
	TInt	count = iGameControlRawInputHandlers->Count();
	for (TInt i=count-1 ; i>=0 && !consumed ; i--)
		consumed=(*iGameControlRawInputHandlers)[i]->ConsumeRawPointerEvent(aPointerEvent);

	if (!consumed)
	{
		count = iGameControlInputHandlers->Count();
		for (i=0 ; i<count ; i++)
			(*iGameControlInputHandlers)[i]->HandlePointerEvent(aPointerEvent);
	}
}



void CGameController::MapToGameEvents(TInt aType, TKeyEvent& aKeyEvent)
{
	if ((aType == EEventKeyDown) || (aType == EEventKeyUp))
	{
//		RDebug::Print(_L("SCAN CODE=%d"), aKeyEvent.iScanCode);
		TGameScanCodeTable*		gameScanCodeEntry = iGameScanCodeTable;
		while (gameScanCodeEntry->iGameScanCode)
		{
			if (gameScanCodeEntry->iPhysicalScanCode == aKeyEvent.iScanCode)
			{
				aKeyEvent.iScanCode = gameScanCodeEntry->iGameScanCode;
				return;
			}
			gameScanCodeEntry++;
		}
	}
	else if (aType == EEventKey)
	{
//		RDebug::Print(_L("KEY CODE=%d"), aKeyEvent.iCode);
		TGameKeyCodeTable*		gameKeyCodeEntry = iGameKeyCodeTable;
		while (gameKeyCodeEntry->iGameKey)
		{
			if (gameKeyCodeEntry->iPhysicalKey == aKeyEvent.iCode)
			{
				aKeyEvent.iCode = gameKeyCodeEntry->iGameKey;
				return;
			}
			gameKeyCodeEntry++;
		}
	}
}


void CGameController::HandleKeyEvent(TInt aType, const TKeyEvent& aKeyEvent)
{
	TKeyEvent	 keyEvent = aKeyEvent;

	MapToGameEvents(aType, keyEvent);

	TBool	consumed = EFalse;
	TInt	count = iGameControlRawInputHandlers->Count();

	for (TInt i=count-1 ; i>=0 && !consumed ; i--)
	{	consumed=(*iGameControlRawInputHandlers)[i]->ConsumeRawKeyEvent(aType, keyEvent);
		if (consumed)
			(*iGameControlInputHandlers)[i]->HandleKeyEvent(aType, keyEvent);
	}

//	RDebug::Print(_L("NOT consumed start"));
	if (!consumed)
	{
		count = iGameControlInputHandlers->Count();
		for (i=0 ; i<count ; i++)
			(*iGameControlInputHandlers)[i]->HandleKeyEvent(aType, keyEvent);
	}
//	RDebug::Print(_L("NOT consumed end"));
}


void CGameController::InjectkeyEvent(TInt aEventType, TInt aKeyOrScanKey, TInt aRepeat)
{
	TKeyEvent	keyEvent;

	keyEvent.iCode = (aEventType == EEventKey) ? aKeyOrScanKey: 0;
	keyEvent.iScanCode = (aEventType != EEventKey) ? aKeyOrScanKey: 0;
	keyEvent.iModifiers = 0;
	keyEvent.iRepeats = aRepeat;

//	RDebug::Print(HERE("type=%d, key=%d"), aEventType, aKeyOrScanKey);

	HandleKeyEvent(aEventType, keyEvent);
}


/******************************************
 *
 * CMameStyleInputHandler implementation
 *
 ******************************************/

CMameStyleInputHandler::CMameStyleInputHandler()
{
}

TBool CMameStyleInputHandler::IsAnyKeyPressed()
{
	return (iNoOfKeyPresses > 0);
}

TBool CMameStyleInputHandler::IsKeyPressed(TInt aKeyCode)
{
#ifdef __WINS__
	RDebug::Print(_L("KEY PRESSED? key=%d"), aKeyCode);
#endif
	TInt presses = 0;

	for (TInt i=0 ; i<MAX_SIMULTANEOUS_KEYPRESSES && presses<iNoOfKeyPresses; i++)
	{
		if (iScanCodes[i])
		{
			presses++;
			if (iKeyCodes[i] == aKeyCode)
			{
				if (DelayKeyUpEvent(iScanCodes[i]))
				{
//					RDebug::Print(_L("Processed DelayKeyUpEvent, scan=%d"), iScanCodes[i]);
					iScanCodes[i] = 0;
					iKeyCodes[i] = 0;
					iNoOfKeyPresses--;
				}
				return ETrue;
			}
		}
	}
	if (iNoIsKeyPressed++ > 30)
	{
		iNoIsKeyPressed = 0;
		ForceDelayedKeyUpEvents();
	}

	return EFalse;
}

void CMameStyleInputHandler::ForceDelayedKeyUpEvents()
{
	TInt presses = 0;
	for (TInt i=0 ; i<MAX_SIMULTANEOUS_KEYPRESSES && presses<iNoOfKeyPresses; i++)
	{
		if (iScanCodes[i])
		{
			presses++;
			if (DelayKeyUpEvent(iScanCodes[i]))
			{
//				RDebug::Print(_L("Processed DelayKeyUpEvent, scan=%d"), iScanCodes[i]);
				iScanCodes[i] = 0;
				iKeyCodes[i] = 0;
				iNoOfKeyPresses--;
			}
		}
	}
}


void CMameStyleInputHandler::HandleKeyEvent(TInt aType, const TKeyEvent& aKeyEvent)
{
	if (aType ==EEventKeyDown)
	{
		iLastScanCode = aKeyEvent.iScanCode;
	}
	else if (aType == EEventKey)
	{
		if (aKeyEvent.iRepeats == 0)
			KeyDown(iLastScanCode, aKeyEvent.iCode);
	}
	else if (aType == EEventKeyUp)
	{
		KeyUp(aKeyEvent.iScanCode);
	}
}

void CMameStyleInputHandler::KeyDown(TInt aScanCode, TInt aKeyCode)
{
	if (iNoOfKeyPresses < MAX_SIMULTANEOUS_KEYPRESSES)
	{
		for (TInt i=0 ; i<MAX_SIMULTANEOUS_KEYPRESSES; i++)
		{
			if (iScanCodes[i] == 0)
			{
				iScanCodes[i] = aScanCode;
				iKeyCodes[i] = aKeyCode;
				iNoOfKeyPresses++;
				break;
			}
		}
	}
	else
	{
		ASSERT(EFalse);
	}
}


TBool CMameStyleInputHandler::DelayKeyUpEvent(TInt aScanCode)
{
	return (aScanCode == GAME_BUTTON1_SCANCODE);
}

void CMameStyleInputHandler::KeyUp(TInt aScanCode)
{
	if (DelayKeyUpEvent(aScanCode))
	{
//		RDebug::Print(_L("DelayKeyUpEvent, scan=%d"), aScanCode);
		return;
	}
	for (TInt i=0 ; i<MAX_SIMULTANEOUS_KEYPRESSES; i++)
	{
		if (iScanCodes[i] == aScanCode)
		{
			iScanCodes[i] = 0;
			iKeyCodes[i] = 0;
			iNoOfKeyPresses--;
			break;
		}
	}
}


/**********************************************
 *
 * TDirectBitmapDrawer
 *
 **********************************************/

void TDirectBitmapDrawer::DrawFrame(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
	if (!iSetup)
	{
		iGameSize = aSrcGameScreenDevice->Size();

		TSize	screenSize = aDstGameScreenDevice->Size();
		TInt	diffX = screenSize.iWidth - iGameSize.Size().iWidth;
		TInt	diffY = screenSize.iHeight - iGameSize.Size().iHeight;

		iGameOrigin.iX = (diffX>0) ? diffX/2 : 0;
		iGameOrigin.iY = (diffY>0) ? diffY/2 : 0;

		iSetup = ETrue;
		iGameVisibleArea = TGameBitmapUtil::VisibleIntersection(screenSize, iGameOrigin, iGameSize.Size(), iGameSize);
	}

	aDstGameScreenDevice->DirectDraw(iGameOrigin, *aSrcGameScreenDevice, iGameSize);
}


TRect TDirectBitmapDrawer::GameVisibleArea() const
{
	return iGameVisibleArea;
}

TSize TDirectBitmapDrawer::GameSize() const
{
	return iGameSize.Size();
}

TPoint TDirectBitmapDrawer::GameOrigin() const
{
	return iGameOrigin;
}

void TDirectBitmapDrawer::SetGameOrigin(const TPoint& aOrigin)
{
	iGameOrigin = aOrigin;
}


/********************************************
 *
 * TScaledBitmapDrawer implementation
 *
 ********************************************/

void TScaledBitmapDrawer::DrawFrame(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
	if (!iScaledBitmapContext.HasSetup())
		iScaledBitmapContext = TScaledBitmapContext(*aDstGameScreenDevice, *aSrcGameScreenDevice, aSrcGameScreenDevice->Size(), ETrue);
	aDstGameScreenDevice->ScaledDrawBitmap(iScaledBitmapContext);
}

TRect TScaledBitmapDrawer::GameVisibleArea() const
{
	return TRect(iScaledBitmapContext.Point(), iScaledBitmapContext.Size());
}

TSize TScaledBitmapDrawer::GameSize() const
{
	return iScaledBitmapContext.Size();
}

TPoint TScaledBitmapDrawer::GameOrigin() const
{
	return iScaledBitmapContext.Point();
}





void TGamePaner::ActivateL(CGameWindow* aGameWindow, CGameController* aGameController, TOrientation aOrientation)
{
	iGameWindow = aGameWindow;
	iGameController = aGameController;
	iOrientation = aOrientation;

	// wait for the first frame to see if it needs paning
	iGameWindow->AddGameScreenDrawObserverL(*this);
}

void TGamePaner::PostDrawFrameToScreen(CGameScreenDevice* /*aDstGameScreenDevice*/)
{
	TRect	rect = iGameWindow->GameVisibleArea();
	TSize	size = iGameWindow->GameSize() - rect.Size();

	if (size.iWidth > 0)
	{
		iDeltaX = 2;
	}
	if (size.iHeight > 0)
	{
		iDeltaY = 2;
	}
	if (iDeltaX || iDeltaY)
	{
		// yup, needs paning, so make sure we get key events
		iGameController->AddGameControlInputHandlerL(*this);
	}

	iCurDelta = iGameWindow->GameOrigin();
	iMaxDelta = size.AsPoint();
	iGameWindow->RemoveGameScreenDrawObserver(*this);
}


void TGamePaner::HandleKeyEvent(TInt aType, const TKeyEvent& aKeyEvent)
{
	TPoint	delta(0, 0);

	if (aType == EEventKey)
	{
		switch (aKeyEvent.iCode)
		{
			case GAME_RIGHT_KEYCODE:
				delta.iX = (iCurDelta.iX + iDeltaX > iMaxDelta.iX) ? iMaxDelta.iX - iCurDelta.iX : iDeltaX;
				break;

			case GAME_LEFT_KEYCODE:
				delta.iX = (iCurDelta.iX < iDeltaX) ? iCurDelta.iX*-1 : iDeltaX*-1;
				break;

			case GAME_UP_KEYCODE:
				if (delta.iY)
					delta.iY = (iCurDelta.iY < iDeltaY) ? iCurDelta.iY*-1 : iDeltaY*-1;
				break;

			case GAME_DOWN_KEYCODE:
				if (delta.iY)
					delta.iY = (iCurDelta.iY + iDeltaY > iMaxDelta.iY) ? iMaxDelta.iY - iCurDelta.iY : iDeltaY;
				break;
		}
	}
	if (delta.iX || delta.iY)
	{
		iCurDelta += delta;
		iGameWindow->SetGameOrigin(iGameWindow->GameOrigin() - delta);
	}
}

/***********************************
 *
 * CInfoMgr implementation
 *
 ***********************************/

#define FCT_VIRTK	0
#define	FCT_SOUND	1
#define FCT_SMPRATE	2
#define	FCT_TURBO	3
#define FCT_FRAMSKP	4
#define FCT_DSPMODE	5
#define FCT_RESET	6
#define FCT_DSK		7
#define FCT_SNA		8
#define FCT_KEYCFG	9
#define FCT_CFGSAV	10

#define DSP_FAST	0
#define DSP_ALT		1
#define DSP_BILIN	2
TInt DspMode = DSP_FAST;

TInt sKeyFunc[MAX_PHONE_KEYS][2]=
{
	//{0x34, FCT_SOUND},
	//{0x35, FCT_DSPMODE},

	{ 0,0 }
};

CInfoMgr::CInfoMgr(CGameWindow& aGameWindow, CGameController& aGameController) : 
	iGameWindow(aGameWindow), iGameController(aGameController), iPosition(10, 0), iSize(150, 12),
	iFgColor(KRgbWhite), iBgColor(KRgbBlack)
{
}

CInfoMgr::~CInfoMgr()
{
	if (iFont)
		iBitmap->FbsBitmapDevice()->ReleaseFont(iFont);
	if (iBitmap)
		delete iBitmap;
}

CInfoMgr* CInfoMgr::NewL(CGameWindow& aGameWindow, CGameController& aGameController)
{
	CInfoMgr*	self = new(ELeave)CInfoMgr(aGameWindow, aGameController);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}

void CInfoMgr::ConstructL()
{
	TTimeIntervalMicroSeconds32	period;
	UserHal::TickPeriod(period);
	iPeriod	= period.Int();

	iBitmap = CFbsGameScreenDevice::NewL(iSize, iGameWindow.GameScreenDevice()->DisplayMode());

	TFontSpec fontSpec(KFontName, 20);
	User::LeaveIfError(iBitmap->FbsBitmapDevice()->GetNearestFontInPixels(iFont, fontSpec));

	CFbsBitGc*	gc = iBitmap->FbsBitGc();
	gc->UseFont(iFont);

	iGameController.AddGameControlInputHandlerL(*this);
}

void CInfoMgr::PostDrawFrameToScreen(CGameScreenDevice* aDstGameScreenDevice)
{
	TPoint p = iGameWindow.GameOrigin() + iPosition;
	if (p.iX < 0)
		p.iX = 0;
	if (p.iY < 0)
		p.iY = 0;

	DrawBitmap();
	aDstGameScreenDevice->DirectDraw(p, *iBitmap, TRect(iSize));
	GameTick();
}


void CInfoMgr::HandleKeyEvent(TInt aType, const TKeyEvent& aKeyEvent)
{
	if (aType == EEventKey)
	{
		for (int i = 0; sKeyFunc[i][0]; i++)
			if (sKeyFunc[i][0] == aKeyEvent.iScanCode)
			{
				iFunction = sKeyFunc[i][1];
				switch(iFunction)
				{
					case FCT_SOUND:
						Sound = 1-Sound;
						break;
					
					case FCT_TURBO:
						Turbo = 1-Turbo;
						break;

					case FCT_FRAMSKP:
						break;
						
					case FCT_DSPMODE:
						DspMode++; DspMode%=3;
						break;
				}
				ActivateL();
			}
		/*if (aKeyEvent.iCode == '#')
		{
			if (!iIsActive)
			{
				ActivateL();
			}
			else
			{
				Deactivate();
			}
		}*/
	}
}


void CInfoMgr::DrawBitmap()
{
	TBuf<32>	text;

	CFbsBitGc*	gc = iBitmap->FbsBitGc();
	gc->SetBrushColor(iBgColor);
	gc->SetBrushStyle(CGraphicsContext::ENullBrush);
	gc->SetPenColor(iFgColor);
	gc->SetPenSize(TSize(1,1));
	gc->Clear();

	//text.Format(_L("%d fps"), iVideoFps);
	switch(iFunction)
	{
		case FCT_SOUND:
			if (Sound)
				text.Format(_L("Sound ON"));
			else
				text.Format(_L("Sound OFF"));
			break;
		case FCT_TURBO:
			if (Turbo)
				text.Format(_L("Turbo ON"));
			else
				text.Format(_L("Turbo OFF"));
			break;
		case FCT_FRAMSKP:
				text.Format(_L("skip %d frames"), iVideoFps);
			break;
		case FCT_DSPMODE:
			switch(DspMode)
			{
				case DSP_FAST:
					text.Format(_L("Display: Fast"));
					break;
				case DSP_ALT:
					text.Format(_L("Display: Flickering"));
					break;
				case DSP_BILIN:
					text.Format(_L("Display: Bilinear"));
					break;
			}
	}
					
	// draw text
	TInt baselineOffset=(iSize.iHeight + iFont->AscentInPixels()) / 2;
	gc->DrawText(text, TRect(iSize), baselineOffset, CGraphicsContext::ECenter);
}

void CInfoMgr::ActivateL()
{
	iIsActive = ETrue;
	iVideoFps = 0;
	iFrames = 0;
	iGameWindow.AddGameScreenDrawObserverL(*this);
}

void CInfoMgr::Deactivate()
{
	iGameWindow.RemoveGameScreenDrawObserver(*this);
	iIsActive = EFalse;
}

extern "C" unsigned long GetTickCount();
static unsigned long OldTime=0;
void CInfoMgr::GameTick()
{
	iFrames++;
	if (iFrames>50)
	{
		Deactivate();
	}
/*
	if (iFrames > 100)
	{
#if 0
		TInt ticks = User::TickCount();
		TInt diff = ticks - iPrevTicks;
		iPrevTicks = ticks;

		TInt time = diff * iPeriod;
		if (!time)
			time = 1;
		iVideoFps = (iFrames * KOneSecondInMicroSeconds) / time;
#else
		unsigned long Time=GetTickCount();
		float gni = (float)(iFrames * KOneSecondInMicroSeconds) / (float)(Time-OldTime);
		iVideoFps = (TInt)gni;
		OldTime = Time;
#endif
		iFrames = 0;
	}*/
}


/**********************************************
 *
 * CFileSelection
 *
 **********************************************/

CFileSelection::CFileSelection(CGameWindow& aGameWindow, CGameController& aGameController) : 
	iGameWindow(aGameWindow), iGameController(aGameController), iPosition(0, 0), iSize(200, 160),
	iFgColor(KRgbWhite), iBgColor(KRgbBlack), iIsActive(EFalse)
{
}

CFileSelection::~CFileSelection()
{
	Destruct();
}



CFileSelection* CFileSelection::NewL(CGameWindow& aGameWindow, CGameController& aGameController, TInt iFileType)
{
	CFileSelection*	self = new(ELeave)CFileSelection(aGameWindow, aGameController);
	CleanupStack::PushL(self);
	self->ConstructL(iFileType);
	CleanupStack::Pop();
	return self;
}

void CFileSelection::ConstructL(TInt iType)
{
	iBitmap = CFbsGameScreenDevice::NewL(iSize, iGameWindow.GameScreenDevice()->DisplayMode());

	TFontSpec fontSpec(KFontName, 15);
	User::LeaveIfError(iBitmap->FbsBitmapDevice()->GetNearestFontInPixels(iFont, fontSpec));

	CFbsBitGc*	gc = iBitmap->FbsBitGc();
	gc->UseFont(iFont);

	iGameController.AddGameControlInputHandlerL(*this);
	iGameWindow.AddScreenOrientationObserverL(*this);
	iGameController.AddGameControlRawInputHandlerL(*this);

	//GetFileList(iFileType);
	iFileType = iType;
}

void CFileSelection::Destruct()
{
	if (iFont)
	{
		iBitmap->FbsBitmapDevice()->ReleaseFont(iFont);
		iFont = NULL;
	}
	if (iBitmap)
	{
		delete iBitmap;
		iBitmap = NULL;
	}
}

TBool CFileSelection::WantDrawToScreen()
{
	return iIsActive;
}

#define NB_LINES	8
void CFileSelection::HandleKeyEvent(TInt aType, const TKeyEvent& aKeyEvent)
{
	if (aType == EEventKey)
	{ //TBool updateDisplay=EFalse;
/*
		if (aKeyEvent.iScanCode == 0x39)
		{
			if (!iIsActive)
				ActivateL();
			else
				Deactivate();

			return;
		}
*/
		if (aKeyEvent.iScanCode == EStdKeyDevice3)
		{
			char szSelectedFileName[256];
			strcpy(szSelectedFileName, szFileNameList[iCurrentIndexInDialog+iFirstIndexInFileList]);

			EjectDiskUPD();
            SetDiskUPD( szSelectedFileName );

			Deactivate();
			return;
		}

		if (iOrientation == EOrientationRotLeft )
		{
			if (aKeyEvent.iScanCode == EStdKeyLeftArrow && iCurrentIndexInDialog<numFiles-1 && iCurrentIndexInDialog<NB_LINES-1)
				iCurrentIndexInDialog++;
			if (aKeyEvent.iScanCode == EStdKeyRightArrow && iCurrentIndexInDialog >0)
				iCurrentIndexInDialog--;
		}
		else
		{
			if (aKeyEvent.iScanCode == EStdKeyRightArrow && iCurrentIndexInDialog<numFiles-1 && iCurrentIndexInDialog<NB_LINES-1)
				iCurrentIndexInDialog++;
			if (aKeyEvent.iScanCode == EStdKeyLeftArrow && iCurrentIndexInDialog >0)
				iCurrentIndexInDialog--;
		}
		if (iCurrentIndexInDialog == NB_LINES-1 && iFirstIndexInFileList+NB_LINES<numFiles)
		{	
			iFirstIndexInFileList++;
			iCurrentIndexInDialog--;
		}
		if (iCurrentIndexInDialog == 0 && iFirstIndexInFileList>0)
			iFirstIndexInFileList--;

		DrawDialogBox();
	}
}

void CFileSelection::ScreenOrientationChanged(TOrientation aOrientation)
{
	iOrientation = aOrientation;

	DrawDialogBox();
}

void CFileSelection::PostDrawFrameToScreen(CGameScreenDevice* aDstGameScreenDevice)
{
	if (iOrientation==EOrientationRotLeft)
		aDstGameScreenDevice->DirectDraw_16bpp_16bppRotLeft(TPoint(170, 5), *iBitmap, TPoint(0, 0), TSize(200, 160));
	else
		aDstGameScreenDevice->DirectDraw_16bpp_16bppRotRight(TPoint(170, 5), *iBitmap, TPoint(0, 0), TSize(200, 160));
}

void CFileSelection::ActivateL()
{
	if (GetFileList(iFileType))
	{
		iIsActive = ETrue;
		iFirstIndexInFileList=0;
		iCurrentIndexInDialog=0;

		iGameWindow.AddGameScreenDrawObserverL(*this);

		DrawDialogBox();
	}
}


void CFileSelection::Deactivate()
{
	iIsActive = EFalse;

	// no longer need to steal pointer events
	iGameWindow.RemoveGameScreenDrawObserver(*this);
	ClearDialog();
}

void CFileSelection::ClearDialog()
{
	CFbsBitGc*	gc = iBitmap->FbsBitGc();
	gc->SetBrushColor(KRgbBlack);
	gc->SetBrushStyle(CGraphicsContext::ENullBrush);
	gc->Clear();
}

extern "C" char DirDisc[ 256 ];
TBool CFileSelection::GetFileList( TInt iFileType )
{
	TBool bFileFound=false;
	RFs fs;
	fs.Connect();
	TFindFile file_finder(fs);
	CDir *file_list;
	numFiles=0;
    
	TBuf<256>	text;
	//TInt err = file_finder.FindWildByDir(aWildName, aPath, file_list);
	TInt err;
	switch(iFileType)
	{
		case FILE_DSK:
		{	//err = file_finder.FindWildByDir(_L("*.dsk"), _L("\\system\\apps\\s60cpc\\Disks\\"), file_list);
			TPtrC8 text8((TUint8*)DirDisc);
			text.Copy(text8);
			err = file_finder.FindWildByDir(_L("*.dsk"), text, file_list);
		}
			break;
		case FILE_SNA:
			err = file_finder.FindWildByDir(_L("*.sna"), _L("\\system\\apps\\s60cpc\\Snap\\"), file_list);
			break;
		case FILE_TAP:
			err = file_finder.FindWildByDir(_L("*.tap"), _L("\\system\\apps\\s60cpc\\Tape\\"), file_list);
			break;
		default:
			return false;
	}

    while (err==KErrNone)
    {
        TInt i;
        for (i=0; i<file_list->Count(); i++)
        {
            TParse fullentry;
            fullentry.Set((*file_list)[i].iName,& file_finder.File(),NULL);

			bFileFound=true;
			for (int j=0;j<fullentry.FullName().Length();j++)
				szFileNameList[numFiles][j] = *(char *)(fullentry.FullName().Ptr()+j);
			szFileNameList[numFiles++][j]=0;

			if (numFiles>MAX_NUM_FILES)
				break;
        }
        delete file_list;
        err=file_finder.FindWild(file_list);
    }

	return bFileFound;
}

void CFileSelection::DrawDialogBox()
{
	CFbsBitGc*	gc = iBitmap->FbsBitGc();
	gc->SetOrientation(ToGraphicsOrientation(iOrientation));
	gc->SetBrushColor(iBgColor);
	gc->SetBrushStyle(CGraphicsContext::ENullBrush);
	gc->Clear();
	gc->SetPenColor(KRgbYellow);
	gc->SetPenSize(TSize(1,1));
	gc->UseFont(iFont);

	TPoint p(5, 10);
	gc->DrawText(_L("Select File"), p);

	p.iY = 42;
	for (TInt i=0; i<NB_LINES; i++)
	{
		if (i+iFirstIndexInFileList>=numFiles)
			break;
		if (i == iCurrentIndexInDialog)
		{
			gc->SetPenColor(KRgbBlue);
		}
		else
		{
			gc->SetPenColor(iFgColor);
		}


		char *szFileName;
		TInt j = strlen(szFileNameList[i+iFirstIndexInFileList])-1;
		while (szFileNameList[i+iFirstIndexInFileList][j] != '\\') j--;
		TPtrC8		text8((TUint8*)(szFileNameList[i+iFirstIndexInFileList]+j+1));
		TBuf<256>	text;
		text.Copy(text8);
		gc->DrawText(text, p);
		p.iY += iFont->HeightInPixels()+2;
	}
}

/******************************************
 *
 * CGameDirectScreenAccess
 *
 ******************************************/

CGameDirectScreenAccess::~CGameDirectScreenAccess()
{
	if (iRequestStatus == KRequestPending)
		iDirectScreenAccess.Cancel();
	if (iDirectScreenAccess.WsHandle() != -1)
		iDirectScreenAccess.Close();

	if (iSemaphore.Handle() != -1)
		iSemaphore.Close();
}

CGameDirectScreenAccess* CGameDirectScreenAccess::NewL(CGameWindow& aGameWindow)
{
	CGameDirectScreenAccess*	self = new(ELeave)CGameDirectScreenAccess(aGameWindow);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}

CGameDirectScreenAccess::CGameDirectScreenAccess(CGameWindow& aGameWindow) :
	iGameWindow(aGameWindow)
{
}


void CGameDirectScreenAccess::ConstructL()
{
	iDirectScreenAccess = RDirectScreenAccess(iGameWindow.iWsSession);
	User::LeaveIfError(iDirectScreenAccess.Construct());
	TBuf<32>	semaphoreName;
	semaphoreName.Format(_L("%S_SEM"), &iGameWindow.iGameName);
	User::LeaveIfError(iSemaphore.CreateGlobal(semaphoreName, 0));

	ClaimScreenL();
}


void CGameDirectScreenAccess::ClaimScreenL()
{
	iRequestStatus = KRequestPending;

	RRegion*	region;
	User::LeaveIfError(iDirectScreenAccess.Request(region, iRequestStatus, iGameWindow.iWsWindow));
	TBool	isFullScreen = region->IsContainedBy(TRect(iGameWindow.iWsWindow.Position(), iGameWindow.iWsWindow.Size()));
	if (!isFullScreen)
		User::Leave(KErrInUse);
}


#if 0
void DumpWindowGroups(RWsSession& aWsSession)
{
	TInt			id = 0;
	TInt			prio = 0;
	TFullName		threadName;
	TFullName		wgName;

	RDebug::Print(_L("************************"));
	while (id >= 0)
	{
		id = aWsSession.FindWindowGroupIdentifier(id , _L("*"));
		if (id > 0)
		{
			TThreadId threadId;
			if (aWsSession.GetWindowGroupClientThreadId(id , threadId) == KErrNone)
			{
				threadName = _L("");
				RThread	thread;
				if (thread.Open(threadId) == KErrNone)
				{
					threadName = thread.Name();
					thread.Close();
				}

				wgName = _L("");
				aWsSession.GetWindowGroupNameFromIdentifier(id, wgName);
				prio = aWsSession.GetWindowGroupOrdinalPriority(id);
			}
			RDebug::Print(_L("id=%d, thread=%S, wg=%S, prio=%d"), id, &threadName, &wgName, prio);
		}
	}
	RDebug::Print(_L("************************"));
}
#endif

void CGameDirectScreenAccess::PauseGameL()
{
	iDirectScreenAccess.Completed();

	User::After(TTimeIntervalMicroSeconds32(1000000));
	iGameWindow.Hide();

//	DumpWindowGroups(iGameWindow.iWsSession);
	iSemaphore.Wait();
	ResumeGameL();
}

void CGameDirectScreenAccess::ResumeGameL()
{
	iGameWindow.Show();
	ClaimScreenL();
	iGameWindow.ClearScreen();
}

void CGameDirectScreenAccess::PreDrawFrameToScreen(CGameScreenDevice* /*aSrcGameScreenDevice*/)
{
	if (iRequestStatus != KRequestPending)
		PauseGameL();
}


//--------------------------------------------------------------------------------------------------
// specific CPC Video functions
//--------------------------------------------------------------------------------------------------
extern "C"
{
	void UpdateScreen( void );
	viddef_t  vid;
}

/*********************************************************
 *
 * Framedrawer
 *
 *********************************************************/
typedef long fixed;			// Our new fixed point type.

#define itofx(x) ((x) << 8)			// Integer to fixed point
#define ftofx(x) ((x) * 256)			// Float to fixed point
#define dtofx(x) ((x) * 256)			// Double to fixed point
#define fxtoi(x) ((x) >> 8)			// Fixed point to integer
#define fxtof(x) ((float) (x) / 256)		// Fixed point to float
#define fxtod(x) ((double)(x) / 256)	        // Fixed point to double
#define Mulfx(x,y) (((y) * (x)) >> 8)		// Multiply a fixed by a fixed
#define Divfx(x,y) (((x) << 8) / (y))    		// Divide a fixed by a fixed

class TCPCFrameDrawer : public MGameScreenDrawer, public MScreenOrientationObserver
{
public:
	~TCPCFrameDrawer();
public:		// implements MGameScreenDrawer
	virtual void	DrawFrame(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice);
	virtual TRect	GameVisibleArea() const;
	virtual TSize	GameSize() const;
	virtual TPoint	GameOrigin() const;
	virtual void	SetGameOrigin(const TPoint& aOrigin);

public:		// implements MScreenOrientationObserver
	virtual void ScreenOrientationChanged(TOrientation aOrientation);

	void BlitImageRotLeft(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice);
	void BlitImageRotRight(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice);
	void BlitImageRotLeftBilinear(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice);
	void PreComputeBilinearGrid(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice);

	TBool		iIsRotLeft;
	
	fixed *fraction_x, *fraction_y, *one_minus_x, *one_minus_y;
    int *ceil_x, *ceil_y, *floor_x, *floor_y;
	int *X,*Y;
};

TCPCFrameDrawer::~TCPCFrameDrawer()
{
	free(fraction_x);
	free(fraction_y);
	free(one_minus_x);
	free(one_minus_y);
	free(ceil_x);
	free(ceil_y);
	free(floor_x);
	free(floor_y);
}

void TCPCFrameDrawer::DrawFrame(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
	if (iIsRotLeft)
	{	if (DspMode==DSP_BILIN)
			BlitImageRotLeftBilinear(aDstGameScreenDevice, aSrcGameScreenDevice);
		else
			BlitImageRotLeft(aDstGameScreenDevice, aSrcGameScreenDevice);
	}
	else
		BlitImageRotRight(aDstGameScreenDevice, aSrcGameScreenDevice);
}

TRect TCPCFrameDrawer::GameVisibleArea() const
{
	return TRect(GameOrigin(), GameSize());
}

TSize TCPCFrameDrawer::GameSize() const
{
	return TSize(176, 208);
}

TPoint	TCPCFrameDrawer::GameOrigin() const
{
#ifdef __WINS__
	return TPoint(0,0);
#else
	return iIsRotLeft ? TPoint(0,0) : TPoint(8,0);
#endif
}

void TCPCFrameDrawer::SetGameOrigin(const TPoint& /*aOrigin*/)
{
}

void TCPCFrameDrawer::PreComputeBilinearGrid(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
	TInt					widthS = aSrcGameScreenDevice->Size().iWidth;
	TInt					heightS = aSrcGameScreenDevice->Size().iHeight;
	TInt					widthD = aDstGameScreenDevice->Size().iWidth;
	TInt					heightD = aDstGameScreenDevice->Size().iHeight;

	fixed nXFactor = Divfx(itofx(widthS),itofx(heightD));
    fixed nYFactor = Divfx(itofx(heightS),itofx(widthD));

	floor_x = (int *)malloc(heightD*sizeof(int));
	floor_y = (int *)malloc(widthD*sizeof(int));
	ceil_x = (int *)malloc(heightD*sizeof(int));
	ceil_y = (int *)malloc(widthD*sizeof(int));
	fraction_x = (fixed *)malloc(heightD*sizeof(fixed));
	fraction_y = (fixed *)malloc(widthD*sizeof(fixed));
	one_minus_x = (fixed *)malloc(heightD*sizeof(fixed));
	one_minus_y = (fixed *)malloc(widthD*sizeof(fixed));
	X = (int *)malloc(heightD*sizeof(int));
	Y = (int *)malloc(widthD*sizeof(int));

	for(int i=0 ; i<widthD ; i++)
	{
        floor_y[i] = (int)fxtoi(Mulfx(itofx(i), nYFactor));
        ceil_y[i] = floor_y[i] + 1;
        if (ceil_y[i] >= heightS) ceil_y[i] = floor_y[i];
        fraction_y[i] = Mulfx(itofx(i), nYFactor) - itofx(floor_y[i]);
        one_minus_y[i] = itofx(1) - fraction_y[i];

		if (fraction_y[i]>128)	// nonbilinear blit
			Y[i]= ceil_y[i];
		else
			Y[i]= floor_y[i];

		for(int j=0 ; j<heightD ; j++)
		{
			floor_x[j] = (int)fxtoi(Mulfx(itofx(j), nXFactor));
            ceil_x[j] = floor_x[j] + 1;
            if (ceil_x[j] >= widthS) ceil_x[j] = floor_x[j];
            fraction_x[j] = Mulfx(itofx(j), nXFactor) - itofx(floor_x[j]);
            one_minus_x[j] = itofx(1) - fraction_x[j];

			if (fraction_x[j]>128)	// nonbilinear blit
				X[j]= ceil_x[j];
			else
				X[j]= floor_x[j];
		}
	}
}

void TCPCFrameDrawer::BlitImageRotLeftBilinear(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
#if 1
	TInt					widthD = aDstGameScreenDevice->Size().iWidth;
	TInt					heightD = aDstGameScreenDevice->Size().iHeight;
	fixed					srcCol;
	int r1,g1,b1,r2,g2,b2,r3,g3,b3,r4,g4,b4;
	TUint8 red, green, blue;
    TUint8 c1, c2;

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(widthD-1, 0));
	TUint8*					srcA = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);

	TDisplayMode displayMode = aDstGameScreenDevice->DisplayMode();
    
	register TUint8* src=srcA;
	for(int i=0 ; i<widthD ; i++)
	{
		TUint16*	dst = frameBufferLine;

		int _floor_y = floor_y[i];
		int _ceil_y = ceil_y[i];
		int _fraction_y = fraction_y[i];
		int _one_minus_y = one_minus_y[i];

		for(int j=0 ; j<heightD ; j++)
		{
			int _fraction_x = fraction_x[j];
			int _one_minus_x = one_minus_x[j];
 
            src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(_floor_y);
			TUint8 fx=src[floor_x[j]];
			TUint8 cx=src[ceil_x[j]];
			
			r1 = RedCPC[fx];
			g1 = GreenCPC[fx];
			b1 = BlueCPC[fx];

            //src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(floor_y);
			r2 = RedCPC[cx];
			g2 = GreenCPC[cx];
			b2 = BlueCPC[cx];

            src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(_ceil_y);
			fx=src[floor_x[j]];
			cx=src[ceil_x[j]];

			r3 = RedCPC[fx];
			g3 = GreenCPC[fx];
			b3 = BlueCPC[fx];

            //src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(_ceil_y);
			r4 = RedCPC[cx];
			g4 = GreenCPC[cx];
			b4 = BlueCPC[cx];
 
            // Blue
            c1 = (TUint8)fxtoi(Mulfx(_one_minus_x, itofx(b1)) + Mulfx(_fraction_x, itofx(b2)));
            c2 = (TUint8)fxtoi(Mulfx(_one_minus_x, itofx(b3)) + Mulfx(_fraction_x, itofx(b4)));
            blue = (TUint8)fxtoi(Mulfx(_one_minus_y, itofx(c1)) + Mulfx(_fraction_y, itofx(c2)));
 
            // Green
            c1 = (TUint8)fxtoi(Mulfx(_one_minus_x, itofx(g1)) + Mulfx(_fraction_x, itofx(g2)));
            c2 = (TUint8)fxtoi(Mulfx(_one_minus_x, itofx(g3)) + Mulfx(_fraction_x, itofx(g4)));
            green = (TUint8)fxtoi(Mulfx(_one_minus_y, itofx(c1)) + Mulfx(_fraction_y, itofx(c2)));
 
            // Red
            c1 = (TUint8)fxtoi(Mulfx(_one_minus_x, itofx(r1)) + Mulfx(_fraction_x, itofx(r2)));
            c2 = (TUint8)fxtoi(Mulfx(_one_minus_x, itofx(r3)) + Mulfx(_fraction_x, itofx(r4)));
            red = (TUint8)fxtoi(Mulfx(_one_minus_y, itofx(c1)) + Mulfx(_fraction_y, itofx(c2)));
 
			TRgb coul(red,green,blue);
			*dst = (TUint16) TGameBitmapUtil::GetColorValue(coul, displayMode);
			dst += aDstGameScreenDevice->BytesPerLine()>>1;
		}
		frameBufferLine--;
	}
#elif 0
	TInt					widthS = aSrcGameScreenDevice->Size().iWidth;
	TInt					heightS = aSrcGameScreenDevice->Size().iHeight;
	TInt					widthD = aDstGameScreenDevice->Size().iWidth;
	TInt					heightD = aDstGameScreenDevice->Size().iHeight;
	fixed					srcCol;
	fixed fraction_x, fraction_y, one_minus_x, one_minus_y;
    int ceil_x, ceil_y, floor_x, floor_y;
	int r1,g1,b1,r2,g2,b2,r3,g3,b3,r4,g4,b4;
	//TRgb c1,c2,c3,c4;
	TUint8 red, green, blue;
    TUint8 c1, c2;

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(widthD-1, 0));
	TUint8*					srcA = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);
//	TUint*					pal = aSrcGameScreenDevice->Palette()->PaletteBuffer();

	TDisplayMode displayMode = aDstGameScreenDevice->DisplayMode();
    
	fixed nXFactor = Divfx(itofx(widthS),itofx(heightD));
    fixed nYFactor = Divfx(itofx(heightS),itofx(widthD));
	
	register TUint8* src=srcA;
	for(int i=0 ; i<widthD ; i++)
	{
		TUint16*	dst = frameBufferLine;
        floor_y = (int)fxtoi(Mulfx(itofx(i), nYFactor));
		for(int j=0 ; j<heightD ; j++)
		{
			floor_x = (int)fxtoi(Mulfx(itofx(j), nXFactor));
            ceil_x = floor_x + 1;
            if (ceil_x >= widthS) ceil_x = floor_x;
            ceil_y = floor_y + 1;
            if (ceil_y >= heightS) ceil_y = floor_y;
            fraction_x = Mulfx(itofx(j), nXFactor) - itofx(floor_x);
            fraction_y = Mulfx(itofx(i), nYFactor) - itofx(floor_y);
            one_minus_x = itofx(1) - fraction_x;
            one_minus_y = itofx(1) - fraction_y;
 
            src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(floor_y);
			r1 = (RgbCPCb[src[floor_x]]>>16) & 0xff;
			g1 = (RgbCPCb[src[floor_x]]>>8) & 0xff;
			b1 = RgbCPCb[src[floor_x]] & 0xff;
			//c1.SetRed(r); c1.SetGreen(g); c1.SetBlue(b);

            src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(floor_y);
			r2 = (RgbCPCb[src[ceil_x]]>>16) & 0xff;
			g2 = (RgbCPCb[src[ceil_x]]>>8) & 0xff;
			b2 = RgbCPCb[src[ceil_x]] & 0xff;
			//c2.SetRed(r); c2.SetGreen(g); c2.SetBlue(b);

            src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(ceil_y);
			r3 = (RgbCPCb[src[floor_x]]>>16) & 0xff;
			g3 = (RgbCPCb[src[floor_x]]>>8) & 0xff;
			b3 = RgbCPCb[src[floor_x]] & 0xff;
			//c3.SetRed(r); c3.SetGreen(g); c3.SetBlue(b);

            src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(ceil_y);
			r4 = (RgbCPCb[src[ceil_x]]>>16) & 0xff;
			g4 = (RgbCPCb[src[ceil_x]]>>8) & 0xff;
			b4 = RgbCPCb[src[ceil_x]] & 0xff;
			//c4.SetRed(r); c4.SetGreen(g); c4.SetBlue(b);
 
            // Blue
            c1 = (TUint8)fxtoi(Mulfx(one_minus_x, itofx(b1)) + Mulfx(fraction_x, itofx(b2)));
            c2 = (TUint8)fxtoi(Mulfx(one_minus_x, itofx(b3)) + Mulfx(fraction_x, itofx(b4)));
            blue = (TUint8)fxtoi(Mulfx(one_minus_y, itofx(c1)) + Mulfx(fraction_y, itofx(c2)));
 
            // Green
            c1 = (TUint8)fxtoi(Mulfx(one_minus_x, itofx(g1)) + Mulfx(fraction_x, itofx(g2)));
            c2 = (TUint8)fxtoi(Mulfx(one_minus_x, itofx(g3)) + Mulfx(fraction_x, itofx(g4)));
            green = (TUint8)fxtoi(Mulfx(one_minus_y, itofx(c1)) + Mulfx(fraction_y, itofx(c2)));
 
            // Red
            c1 = (TUint8)fxtoi(Mulfx(one_minus_x, itofx(r1)) + Mulfx(fraction_x, itofx(r2)));
            c2 = (TUint8)fxtoi(Mulfx(one_minus_x, itofx(r3)) + Mulfx(fraction_x, itofx(r4)));
            red = (TUint8)fxtoi(Mulfx(one_minus_y, itofx(c1)) + Mulfx(fraction_y, itofx(c2)));
 
			TRgb coul(red,green,blue);
			*dst = (TUint16) TGameBitmapUtil::GetColorValue(coul, displayMode);
			dst += aDstGameScreenDevice->BytesPerLine()>>1;
			//srcCol += (float)widthS/(float)heightD;
		}
		//src = srcA + widthS*(TUint8)((float)(i+1)*(float)heightS/(float)widthD);
		frameBufferLine--;
	}
#else
	TInt					widthS = aSrcGameScreenDevice->Size().iWidth;
	TInt					heightS = aSrcGameScreenDevice->Size().iHeight;
	TInt					widthD = aDstGameScreenDevice->Size().iWidth;
	TInt					heightD = aDstGameScreenDevice->Size().iHeight;
	float					srcCol;
	float fraction_x, fraction_y, one_minus_x, one_minus_y;
    int ceil_x, ceil_y, floor_x, floor_y;
	int r,g,b;
	TRgb c1,c2,c3,c4;
	TUint8 red, green, blue;
    TUint8 b1, b2;

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(widthD-1, 0));
	TUint8*					srcA = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);
//	TUint*					pal = aSrcGameScreenDevice->Palette()->PaletteBuffer();

	TDisplayMode displayMode = aDstGameScreenDevice->DisplayMode();
    
	float nXFactor = (float)widthS/(float)heightD;
    float nYFactor = (float)heightS/(float)widthD;
	
	register TUint8* src=srcA;
	for(int i=0 ; i<widthD ; i++)
	{
		TUint16*	dst = frameBufferLine;
		for(int j=0 ; j<heightD ; j++)
		{
			floor_x = (int)floor(j * nXFactor);
            floor_y = (int)floor(i * nYFactor);
            ceil_x = floor_x + 1;
            if (ceil_x >= widthS) ceil_x = floor_x;
            ceil_y = floor_y + 1;
            if (ceil_y >= heightS) ceil_y = floor_y;
            fraction_x = j * nXFactor - floor_x;
            fraction_y = i * nYFactor - floor_y;
            one_minus_x = 1.0 - fraction_x;
            one_minus_y = 1.0 - fraction_y;
 
			//c1 = bTemp.GetPixel(floor_x, floor_y);
            src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(floor_y);
			r = (RgbCPCb[src[floor_x]]>>16) & 0xff;
			g = (RgbCPCb[src[floor_x]]>>8) & 0xff;
			b = RgbCPCb[src[floor_x]] & 0xff;
			c1.SetRed(r); c1.SetGreen(g); c1.SetBlue(b);

            //c2 = bTemp.GetPixel(ceil_x, floor_y);
            src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(floor_y);
			r = (RgbCPCb[src[ceil_x]]>>16) & 0xff;
			g = (RgbCPCb[src[ceil_x]]>>8) & 0xff;
			b = RgbCPCb[src[ceil_x]] & 0xff;
			c2.SetRed(r); c2.SetGreen(g); c2.SetBlue(b);

            //c3 = bTemp.GetPixel(floor_x, ceil_y);
            src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(ceil_y);
			r = (RgbCPCb[src[floor_x]]>>16) & 0xff;
			g = (RgbCPCb[src[floor_x]]>>8) & 0xff;
			b = RgbCPCb[src[floor_x]] & 0xff;
			c3.SetRed(r); c3.SetGreen(g); c3.SetBlue(b);

            //c4 = bTemp.GetPixel(ceil_x, ceil_y);
            src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(ceil_y);
			r = (RgbCPCb[src[ceil_x]]>>16) & 0xff;
			g = (RgbCPCb[src[ceil_x]]>>8) & 0xff;
			b = RgbCPCb[src[ceil_x]] & 0xff;
			c4.SetRed(r); c4.SetGreen(g); c4.SetBlue(b);
 
            // Blue
            b1 = (TUint8)(one_minus_x * c1.Blue() + fraction_x * c2.Blue());
            b2 = (TUint8)(one_minus_x * c3.Blue() + fraction_x * c4.Blue());
            blue = (TUint8)(one_minus_y * (float)(b1) + fraction_y * (float)(b2));
 
            // Green
            b1 = (TUint8)(one_minus_x * c1.Green() + fraction_x * c2.Green());
            b2 = (TUint8)(one_minus_x * c3.Green() + fraction_x * c4.Green());
            green = (TUint8)(one_minus_y * (float)(b1) + fraction_y * (float)(b2));
 
            // Red
            b1 = (TUint8)(one_minus_x * c1.Red() + fraction_x * c2.Red());
            b2 = (TUint8)(one_minus_x * c3.Red() + fraction_x * c4.Red());
            red = (TUint8)(one_minus_y * (float)(b1) + fraction_y * (float)(b2));
 
			TRgb coul(red,green,blue);
			*dst = (TUint16) TGameBitmapUtil::GetColorValue(coul, displayMode);
			dst += aDstGameScreenDevice->BytesPerLine()>>1;
			//srcCol += (float)widthS/(float)heightD;
		}
		//src = srcA + widthS*(TUint8)((float)(i+1)*(float)heightS/(float)widthD);
		frameBufferLine--;
	}
#endif
}

void TCPCFrameDrawer::BlitImageRotLeft(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
#if 1
	TInt					widthS = aSrcGameScreenDevice->Size().iWidth;
	TInt					widthD = aDstGameScreenDevice->Size().iWidth;
	TInt					heightD = aDstGameScreenDevice->Size().iHeight;

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(widthD-1, 0));
	TUint8*					srcA = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);
	TUint*					pal = aSrcGameScreenDevice->Palette()->PaletteBuffer();

	int *_X,*_Y;
	static gni=0;
	if (DspMode==DSP_ALT)
	{
		if (gni)
		{
			_X=floor_x;
			_Y=floor_y;
		}
		else
		{
			_X=ceil_x;
			_Y=ceil_y;
		}
		gni=1-gni;
	}
	else
	{
		_X=X;
		_Y=Y;
	}

	TInt Inc = aDstGameScreenDevice->BytesPerLine()>>1;
	register TUint8* src=srcA;
	for(int i=0 ; i<widthD ; i++)
	{
		src = srcA + widthS * _Y[i];
		TUint16*	dst = frameBufferLine;
		for(int j=0 ; j<heightD ; j++)
		{
			*dst = (TUint16) pal[src[_X[j]]];
			dst += Inc;
		}
		frameBufferLine--;
	}
#elif 0
	TInt					widthS = aSrcGameScreenDevice->Size().iWidth;
	TInt					heightS = aSrcGameScreenDevice->Size().iHeight;
	TInt					widthD = aDstGameScreenDevice->Size().iWidth;
	TInt					heightD = aDstGameScreenDevice->Size().iHeight;
	fixed					srcCol;
	fixed Ratio1 = Divfx(itofx(widthS), itofx(heightD));
	fixed Ratio2 = Divfx(itofx(heightS), itofx(widthD));

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(widthD-1, 0));
	TUint8*					srcA = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);
	TUint*					pal = aSrcGameScreenDevice->Palette()->PaletteBuffer();

	register TUint8* src=srcA;
	for(int i=0 ; i<widthD ; i++)
	{
		srcCol = 0;
		TUint16*	dst = frameBufferLine;
		for(int j=0 ; j<heightD ; j++)
		{
			*dst = (TUint16) pal[src[(TUint16)fxtoi(srcCol)]];
			dst += aDstGameScreenDevice->BytesPerLine()>>1;
			srcCol += Ratio1;
		}
		src = srcA + widthS*(TUint16)fxtoi(Mulfx(itofx(i+1), Ratio2));
		frameBufferLine--;
	}
#else
	TInt					widthS = aSrcGameScreenDevice->Size().iWidth;
	TInt					heightS = aSrcGameScreenDevice->Size().iHeight;
	TInt					widthD = aDstGameScreenDevice->Size().iWidth;
	TInt					heightD = aDstGameScreenDevice->Size().iHeight;
	float					srcCol;

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(widthD-1, 0));
	TUint8*					srcA = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);
	TUint*					pal = aSrcGameScreenDevice->Palette()->PaletteBuffer();

	register TUint8* src=srcA;
	for(int i=0 ; i<widthD ; i++)
	{
		srcCol = 0.0f;
		TUint16*	dst = frameBufferLine;
		for(int j=0 ; j<heightD ; j++)
		{
			*dst = (TUint16) pal[src[(TUint16)srcCol]];
			dst += aDstGameScreenDevice->BytesPerLine()>>1;
			srcCol += (float)widthS/(float)heightD;
		}
		src = srcA + widthS*(TUint16)((float)(i+1)*(float)heightS/(float)widthD);
		frameBufferLine--;
	}
#endif
}

void TCPCFrameDrawer::BlitImageRotRight(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
	TInt					width = aSrcGameScreenDevice->Size().iWidth;
	TInt					height = aSrcGameScreenDevice->Size().iHeight;

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(8, width-1));
	register TUint8*		src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);
	TUint*					pal = aSrcGameScreenDevice->Palette()->PaletteBuffer();

	for(int i=0 ; i<height ; i++)
	{
		TUint16*	dst = frameBufferLine;
		for(int j=0 ; j<width ; j++)
		{
			*dst = (TUint16) pal[*src++];
			dst -= aDstGameScreenDevice->BytesPerLine()>>1;
		}
		frameBufferLine++;
	}
}


void TCPCFrameDrawer::ScreenOrientationChanged(TOrientation aOrientation)
{
	iIsRotLeft = aOrientation == EOrientationRotLeft;
}

/****************************************************
 *
 * IMPLEMENTATION OF VIDEO ROUTINES
 *
 ****************************************************/

static CGameWindow*				sGameWindow = NULL;
CMemGameScreenDevice*			sMemGameScreenDevice = NULL;		// Emulator's framebuffer
CGamePalette*					sCGamePalette = NULL;
TCPCFrameDrawer					sCPCFrameDrawer;

#define MAXWINMODES 2
static int windowedModes[MAXWINMODES][2] = 
{
	{320, 266},
	{320, 266}
};

static char* windowedModesText[MAXWINMODES] = 
{
	"320x266 L",
	"320x266 R"
};



int VID_NumModes(void)
{
    return MAXWINMODES;
}

static int oldMode = -1;

int VID_SetMode(int modenum)
{
	if (modenum >= MAXWINMODES) 
	{
		CPCDebugPrint("VID_SetMode modenum >= MAXWINMODES\n");
		return -1;
	}
	if (oldMode == modenum)
		return 1;

	
	/*
	 * First time
	 */
	TBool	isRotLeft = (modenum == 0);
	if (oldMode == -1)
	{
		TSize	gameSize(windowedModes[modenum][0], windowedModes[modenum][1]);
		sGameWindow = CGameWindow::NewL(_L("S60CPC"));
		sGameWindow->SetGameScreenDrawer(&sCPCFrameDrawer);
		sGameWindow->AddScreenOrientationObserverL(sCPCFrameDrawer);
		sGameWindow->GameController().AddGameControlInputHandlerL(sCPCControlInputHandler);
		sGameWindow->AddGameScreenDrawObserverL(sCPCControlInputHandler);
		sGameWindow->AddScreenOrientationObserverL(sCPCControlInputHandler);

		CVirtualKeyboard&		virtualKeyboard = sGameWindow->VirtualKeyboard();
		virtualKeyboard.SetKeyCodesAreScanCodes(ETrue);
		virtualKeyboard.SetPosition(TPoint(0, 50));

		int vidbuffersize = gameSize.iWidth * gameSize.iHeight;

		vid.width  = gameSize.iWidth;
		vid.height = gameSize.iHeight;

		vid.direct = vid.buffer = (unsigned char*) malloc(vidbuffersize);
		vid.rowbytes = vid.width;
		vid.bpp      = 1;
		vid.recalc   = 1;

		sMemGameScreenDevice = CMemGameScreenDevice::NewL(TSize(vid.width, vid.height), EColor256, (TUint8*) vid.direct);

		sCPCFrameDrawer.PreComputeBilinearGrid(sGameWindow->GameScreenDevice(), sMemGameScreenDevice);

		virtualKeyboard.SetHotKeyForVirtualKeyboard(KVirtualKeyboardHotkey);
	}
	else
	{
		sGameWindow->ClearScreen();
		//SaveRotationSetting(isRotLeft);
	}

	oldMode = modenum;
	sGameWindow->SetScreenOrientation(isRotLeft ? EOrientationRotLeft : EOrientationRotRight);
	vid.modenum  = modenum;

	CPCDebugPrint("VID_SetMode(%d) DONE\n", modenum);
	return 1;
}

void VID_SetPalette ()
{
	//ASSERT(sMemGameScreenDevice);

	TDisplayMode	displayMode = sGameWindow->GameScreenDevice()->DisplayMode();
	if (!sCGamePalette)
		sCGamePalette = CGamePalette::NewL(displayMode);

	int i;
	for( i=0 ; i<32 ; i++)
	{
		int r = (RgbCPC[i]>>16) & 0xff;
		int g = (RgbCPC[i]>>8) & 0xff;
		int b = RgbCPC[i] & 0xff;

		TRgb	color(r, g, b);
		(sCGamePalette->PaletteBuffer())[i] = (TUint16) TGameBitmapUtil::GetColorValue(color, displayMode);
	}
	sMemGameScreenDevice->SetPalette(sCGamePalette);

	for ( i = 0; i < 32; i++ )
	    RgbCPC[ i ] = i;

	for (i=0;i<16;i++)
		clav[i] = 0xFF;
}

void UpdateScreen( void )
{
	// Affiche le framebuffer.
	/*static TInt sCount = 0;
	if (sCount++ == 3)
	{
		sCount = 0;
		return;
	}*/
	sGameWindow->GameFrameTick(sMemGameScreenDevice);
}

#ifdef USE_SOUND
extern CPolledActiveScheduler *sTheScheduler;
extern CGameAudio *sGameAudio;
#endif

void Quit(void)
{
	delete sGameWindow;
	delete sMemGameScreenDevice;
	delete sCGamePalette;

#ifdef USE_SOUND
	delete sGameAudio;
	delete sTheScheduler;
#endif

#ifdef __MARM__
	RProcess	me;
	me.Terminate(0);
#else
	RThread	me;
	me.Terminate(0);
//	exit(0);
#endif
}


// *********************************************************************************************************************************

/**********************************************
 *
 * CMenuPage
 *
 **********************************************/

int iSoundSampleRate = 3;
#define	MAXBUFISIZE		6558	// 2*4 + MAX_PHONE_KEYS*(sizeof(stCharMapping)+sizeof(stDirectCommand))

TInt posInBuffer=0;
void writeBuffer(TUint8 *buffer, TUint8 *src, TInt size)
{
	memcpy(buffer+posInBuffer,src,size);
	posInBuffer+=size;
}

void readBuffer(TUint8 *buffer, TUint8 *dest, TInt size)
{
	memcpy(dest,buffer+posInBuffer,size);
	posInBuffer+=size;
}

void SaveConfig()
{
	RFile file;
	TInt size = 0;
	TUint8 cBuffer[MAXBUFISIZE];
	
	posInBuffer=0;

	RFs fs;
	fs.Connect();

#ifdef __WINS__
	file.Replace(fs,_L("cpcconfig.ini"),EFileWrite);
#else
	file.Replace(fs,_L("e:\\system\\apps\\s60cpc\\cpcconfig.ini"),EFileWrite);
#endif
	writeBuffer(cBuffer,(TUint8 *)&Sound,sizeof(int));
	writeBuffer(cBuffer,(TUint8 *)&iSoundSampleRate,sizeof(int));
	writeBuffer(cBuffer,(TUint8 *)&Turbo,sizeof(int));
	writeBuffer(cBuffer,(TUint8 *)&DspMode,sizeof(int));
	writeBuffer(cBuffer,(TUint8 *)&SKIPFRAME,sizeof(int));
	size = 5*sizeof(int);

	for (int i=0;sKeyMapTable[i].Char;i++);
	size += sizeof(int)+(i+1)*sizeof(stCharMapping);
	writeBuffer(cBuffer,(TUint8 *)&i,sizeof(int));
	writeBuffer(cBuffer,(TUint8 *)sKeyMapTable,sizeof(stCharMapping)*(i+1));

	for (int j=0;trDirectCommands[j].iStdScanCode;j++);
	size += sizeof(int)+(j+1)*sizeof(stDirectCommand);
	writeBuffer(cBuffer,(TUint8 *)&j,sizeof(int));
	writeBuffer(cBuffer,(TUint8 *)trDirectCommands,sizeof(stDirectCommand)*(j+1));

	TBuf8<MAXBUFISIZE> iBuffer;

	iBuffer.Copy(cBuffer, size);

	file.Write(iBuffer);

	file.Flush();
	file.Close();
}

void ReadConfig()
{
	RFile file;
	TUint8 cBuffer[MAXBUFISIZE];
	TBuf8<MAXBUFISIZE> iBuffer;
	
	posInBuffer=0;

	RFs fs;
	fs.Connect();

#ifdef __WINS__
	if (file.Open(fs,_L("cpcconfig.ini"),EFileRead)==KErrNone)
#else
	if (file.Open(fs,_L("e:\\system\\apps\\s60cpc\\cpcconfig.ini"),EFileRead)==KErrNone)
#endif
	{
		file.Read(iBuffer);

		file.Close();

		for (int i=0;i<iBuffer.Size();i++)
			cBuffer[i] = iBuffer[i];
		
		readBuffer(cBuffer,(TUint8 *)&Sound,sizeof(int));
		readBuffer(cBuffer,(TUint8 *)&iSoundSampleRate,sizeof(int));
		readBuffer(cBuffer,(TUint8 *)&Turbo,sizeof(int));
		readBuffer(cBuffer,(TUint8 *)&DspMode,sizeof(int));
		readBuffer(cBuffer,(TUint8 *)&SKIPFRAME,sizeof(int));

		readBuffer(cBuffer,(TUint8 *)&i, sizeof(int));
		readBuffer(cBuffer,(TUint8 *)sKeyMapTable,sizeof(stCharMapping)*(i+1));

		readBuffer(cBuffer,(TUint8 *)&i,sizeof(int));
		readBuffer(cBuffer,(TUint8 *)trDirectCommands,sizeof(stDirectCommand)*(i+1));
	}
}


CMenuPage::CMenuPage(CGameWindow& aGameWindow, CGameController& aGameController) : 
	iGameWindow(aGameWindow), iGameController(aGameController), iPosition(0, 0), iSize(176, 208),
	iFgColor(KRgbWhite), iBgColor(KRgbBlack), iIsActive(EFalse), iConfigKeyboard(0)
{
}

CMenuPage::~CMenuPage()
{
	Destruct();
}



CMenuPage* CMenuPage::NewL(CGameWindow& aGameWindow, CGameController& aGameController, CFileSelection *iFileSel, CVirtualKeyboard *iVK)
{
	CMenuPage*	self = new(ELeave)CMenuPage(aGameWindow, aGameController);
	CleanupStack::PushL(self);
	self->ConstructL(iFileSel, iVK);
	CleanupStack::Pop();
	return self;
}

void CMenuPage::ConstructL(CFileSelection *_iFileSel, CVirtualKeyboard *_iVK)
{
	iBitmap = CFbsGameScreenDevice::NewL(iSize, iGameWindow.GameScreenDevice()->DisplayMode());

	TFontSpec fontSpec(KFontName, 15);
	User::LeaveIfError(iBitmap->FbsBitmapDevice()->GetNearestFontInPixels(iFont, fontSpec));

	CFbsBitGc*	gc = iBitmap->FbsBitGc();
	gc->UseFont(iFont);

	iGameController.AddGameControlInputHandlerL(*this);
	iGameWindow.AddScreenOrientationObserverL(*this);
	iGameController.AddGameControlRawInputHandlerL(*this);

	iFileSelectionDSK = _iFileSel;
	iVK=_iVK;
}

void CMenuPage::Destruct()
{
	if (iFont)
	{
		iBitmap->FbsBitmapDevice()->ReleaseFont(iFont);
		iFont = NULL;
	}
	if (iBitmap)
	{
		delete iBitmap;
		iBitmap = NULL;
	}
}

TBool CMenuPage::WantDrawToScreen()
{
	return iIsActive;
}

void CMenuPage::SelectItem()
{
	switch(iCurrentIndexInDialog)
	{
		case FCT_SOUND:
			Sound = 1 - Sound;
			break;
		case FCT_TURBO:
			Turbo = 1 - Turbo;
			break;
		case FCT_FRAMSKP:
			SKIPFRAME++; SKIPFRAME%=10;
			break;

		case FCT_DSPMODE:
			DspMode++; DspMode%=3;
			break;

		case FCT_RESET:
            WriteVGA( 0x89 );
            ResetZ80();
			break;

		case FCT_DSK:
			iFileSelectionDSK->ActivateL();
			break;

		case FCT_VIRTK:
			iVK->ActivateL();
			break;

		case FCT_KEYCFG:
			iConfigKeyboard = 1;
			break;

		case FCT_CFGSAV:
			SaveConfig();
			break;

		case FCT_SMPRATE:
			iSoundSampleRate++; iSoundSampleRate%=7;
			break;
	}
}

#define NB_MENU_LINES	11
void CMenuPage::HandleKeyEvent(TInt aType, const TKeyEvent& aKeyEvent)
{
	if (aType == EEventKey)
	{ 
		if (aKeyEvent.iScanCode == EStdKeyDevice0 && !iIsActive)
		{
			ActivateL();

			return;
		}
		
		if (!iIsActive)
			return;

		if (iConfigKeyboard==1)
		{
			if (aKeyEvent.iScanCode != EStdKeyDevice0 && aKeyEvent.iScanCode != EStdKeyDevice1)
			{
				iKeyScanCode = aKeyEvent.iScanCode;
				iConfigKeyboard=2;
				iVK->ActivateL();
			}
		}
		else
		{
			if (aKeyEvent.iScanCode == EStdKeyDevice1)
			{
				Deactivate();

				return;
			}

			if (aKeyEvent.iScanCode == EStdKeyDevice3 || aKeyEvent.iScanCode == EStdKeyDevice0)
			{
				if (iCurrentIndexInDialog == FCT_VIRTK || (iCurrentIndexInDialog>FCT_DSPMODE && iCurrentIndexInDialog<FCT_KEYCFG))
					Deactivate();

				SelectItem();

				if (iCurrentIndexInDialog == FCT_VIRTK || (iCurrentIndexInDialog>FCT_DSPMODE && iCurrentIndexInDialog<FCT_KEYCFG))
					return;
			}

			if (aKeyEvent.iScanCode == EStdKeyDownArrow)
				if (iCurrentIndexInDialog<NB_MENU_LINES-1)
					iCurrentIndexInDialog++;
				else
					iCurrentIndexInDialog = 0;
			if (aKeyEvent.iScanCode == EStdKeyUpArrow)
				if (iCurrentIndexInDialog >0)
					iCurrentIndexInDialog--;
				else
					iCurrentIndexInDialog = NB_MENU_LINES-1;

			/*if (iOrientation == EOrientationRotLeft )
			{
				if (aKeyEvent.iScanCode == EStdKeyLeftArrow && iCurrentIndexInDialog<NB_MENU_LINES-1)
					iCurrentIndexInDialog++;
				if (aKeyEvent.iScanCode == EStdKeyRightArrow && iCurrentIndexInDialog >0)
					iCurrentIndexInDialog--;
			}
			else
			{
				if (aKeyEvent.iScanCode == EStdKeyRightArrow && iCurrentIndexInDialog<NB_MENU_LINES-1)
					iCurrentIndexInDialog++;
				if (aKeyEvent.iScanCode == EStdKeyLeftArrow && iCurrentIndexInDialog >0)
					iCurrentIndexInDialog--;
			}*/
		}

		DrawDialogBox();
	}
}

void CMenuPage::ScreenOrientationChanged(TOrientation aOrientation)
{
	iOrientation = aOrientation;

	DrawDialogBox();
}

void CMenuPage::PostDrawFrameToScreen(CGameScreenDevice* aDstGameScreenDevice)
{
	aDstGameScreenDevice->DirectDraw(TPoint(0, 0), *iBitmap, TSize(176, 208));
	/*if (iOrientation==EOrientationRotLeft)
		aDstGameScreenDevice->DirectDraw_16bpp_16bppRotLeft(TPoint(170, 5), *iBitmap, TPoint(0, 0), TSize(200, 160));
	else
		aDstGameScreenDevice->DirectDraw_16bpp_16bppRotRight(TPoint(170, 5), *iBitmap, TPoint(0, 0), TSize(200, 160));*/

	if (iConfigKeyboard==2 && !iVK->iIsActive)
	{
		char szCmd[256];
		if (iVK->GetCommandString(szCmd))
		{	for (TInt i=0;i<strlen(szCmd);i++)
				szCmd[i]=(char)toupper(szCmd[i]);

			if (strlen(szCmd)==1)	// 1 touche
			{
				for (int i=0;sKeyMapTable[i].Char;i++)
				{
					if (sKeyMapTable[i].Char == iKeyScanCode)
					{
						for (int j=0;j<NUM_MAPPED_CHARS;j++)
						{
							if (trChar[j].Char == szCmd[0])
							{	sKeyMapTable[i].Ligne = trChar[j].Ligne;
								sKeyMapTable[i].Port = trChar[j].Port;
								break;
							}
						}
						break;
					}
				}
				if (!sKeyMapTable[i].Char)
				{
					sKeyMapTable[i].Char = iKeyScanCode;

					for (int j=0;j<NUM_MAPPED_CHARS;j++)
					{
						if (trChar[j].Char == szCmd[0])
						{	sKeyMapTable[i].Ligne = trChar[j].Ligne;
							sKeyMapTable[i].Port = trChar[j].Port;
							break;
						}
					}
				}
			}
			else					// 1 commande
			{
				for (int i=0;trDirectCommands[i].iStdScanCode;i++)
				{
					if (trDirectCommands[i].iStdScanCode == iKeyScanCode)
					{
						strcpy(trDirectCommands[i].szCmdString,szCmd);
						break;
					}
				}
				if (!trDirectCommands[i].iStdScanCode)
				{
					trDirectCommands[i].iStdScanCode = iKeyScanCode;
					strcpy(trDirectCommands[i].szCmdString,szCmd);
				}
			}
		}
		else	// Remove a key map
		{
			for (int i=0;sKeyMapTable[i].Char;i++)
			{
				if (sKeyMapTable[i].Char == iKeyScanCode)
				{
					for (int j=i;sKeyMapTable[j].Char;j++)
					{
						memcpy(sKeyMapTable+j,sKeyMapTable+j+1,sizeof(stCharMapping));
					}
					break;
				}
			}
			for (i=0;trDirectCommands[i].iStdScanCode;i++)
			{
				if (trDirectCommands[i].iStdScanCode == iKeyScanCode)
				{
					for (int j=i;trDirectCommands[j].iStdScanCode;j++)
						memcpy(trDirectCommands+j,trDirectCommands+j+1,sizeof(stDirectCommand));
					break;
				}
			}
		}
		iConfigKeyboard = 0;
		DrawDialogBox();
	}

}

void CMenuPage::ActivateL()
{
	iIsActive = ETrue;
	iCurrentIndexInDialog=0;

	iGameWindow.AddGameScreenDrawObserverL(*this);

	DrawDialogBox();
}


void CMenuPage::Deactivate()
{
	iIsActive = EFalse;

	// no longer need to steal pointer events
	iGameWindow.RemoveGameScreenDrawObserver(*this);
	ClearDialog();
}

void CMenuPage::ClearDialog()
{
	CFbsBitGc*	gc = iBitmap->FbsBitGc();
	gc->SetBrushColor(KRgbBlack);
	gc->SetBrushStyle(CGraphicsContext::ENullBrush);
	gc->Clear();
}

void CMenuPage::DrawItem(TInt i, CFbsBitGc *gc, TBuf<256> text, TPoint *p)
{
	if (i == iCurrentIndexInDialog)
	{
		gc->SetPenColor(KRgbYellow);
	}
	else
	{
		gc->SetPenColor(iFgColor);
	}
	gc->DrawText(text, *p);
	p->iY += iFont->HeightInPixels()+2;
}

void CMenuPage::DrawDialogBox()
{
	CFbsBitGc*	gc = iBitmap->FbsBitGc();
	gc->SetOrientation(ToGraphicsOrientation(iOrientation));
	gc->SetBrushColor(iBgColor);
	gc->SetBrushStyle(CGraphicsContext::ENullBrush);
	gc->Clear();
	gc->SetPenColor(KRgbYellow);
	gc->SetPenSize(TSize(1,1));
	gc->UseFont(iFont);

	TBuf<256>	text;
	TPoint p(5, 20);

	if (!iConfigKeyboard)
	{
		text.Format(_L("Virtual Keyboard"), 0);
		DrawItem(FCT_VIRTK, gc, text, &p);

		if (Sound)
			text.Format(_L("Sound ON"));
		else
			text.Format(_L("Sound OFF"));
		DrawItem(FCT_SOUND, gc, text, &p);
		
		switch(iSoundSampleRate)
		{
			case 0:
				text.Format(_L("Sample rate: 8000 Hz"));
				break;
			case 1:
				text.Format(_L("Sample rate: 11025 Hz"));
				break;
			case 2:
				text.Format(_L("Sample rate: 16000 Hz"));
				break;
			case 3:
				text.Format(_L("Sample rate: 22050 Hz"));
				break;
			case 4:
				text.Format(_L("Sample rate: 32000 Hz"));
				break;
			case 5:
				text.Format(_L("Sample rate: 44100 Hz"));
				break;
			case 6:
				text.Format(_L("Sample rate: 48000 Hz"));
				break;
		}
		DrawItem(FCT_SMPRATE, gc, text, &p);

		if (Turbo)
			text.Format(_L("Turbo ON"));
		else
			text.Format(_L("Turbo OFF"));
		DrawItem(FCT_TURBO, gc, text, &p);
		
		text.Format(_L("skip %d frames"), SKIPFRAME);
		DrawItem(FCT_FRAMSKP, gc, text, &p);

		switch(DspMode)
		{
			case DSP_FAST:
				text.Format(_L("Display: Fast"));
				break;
			case DSP_ALT:
				text.Format(_L("Display: Flickering"));
				break;
			case DSP_BILIN:
				text.Format(_L("Display: Bilinear"));
				break;
		}
		DrawItem(FCT_DSPMODE, gc, text, &p);

		text.Format(_L("Reset"), 0);
		DrawItem(FCT_RESET, gc, text, &p);

		text.Format(_L("Select Disc"), 0);
		DrawItem(FCT_DSK, gc, text, &p);

		text.Format(_L("Snapshot"), 0);
		DrawItem(FCT_SNA, gc, text, &p);

		text.Format(_L("Config Keys"), 0);
		DrawItem(FCT_KEYCFG, gc, text, &p);

		text.Format(_L("Save config"), 0);
		DrawItem(FCT_CFGSAV, gc, text, &p);

		gc->SetPenColor(KRgbCyan);
		p.iY = 200;
		text.Format(_L("Select"), 0);
		gc->DrawText(text, p);
		p.iX = 145;
		text.Format(_L("Back"), 0);
		gc->DrawText(text, p);
	}
	else
	{
		p.iY = 120;
		text.Format(_L("Press a phone key"));
		DrawItem(FCT_RESET, gc, text, &p);
	}
}
